<?php
/* by Tomasz 'Devilshakerz' Mlynski [devilshakerz.com]; Copyright (C) 2014-2016
 released under Creative Commons BY-NC-SA 4.0 license: https://creativecommons.org/licenses/by-nc-sa/4.0/ */

namespace DVZMentions;

class Alerts
{
    static $locations = [];

    static function registerLocation($location)
    {
        self::$locations[] = $location;
    }

    static function myalertsRegisterFormatters()
    {
        global $mybb, $lang;

        if ($mybb->user['uid']) {

            $formatterManager = \MybbStuff_MyAlerts_AlertFormatterManager::getInstance();

            foreach (self::$locations as $location) {
                $class = 'MybbStuff_MyAlerts_Formatter_DVZMentions' . ucfirst($location) . 'Formatter';
                $formatterManager->registerFormatter(new $class($mybb, $lang, 'dvz_mentions_' . $location));
            }

        }
    }

    static function adminPlugins()
    {
        global $mybb, $lang;

        if (self::$locations) {

            $lang->load('dvz_mentions');

            $alertTypeManager = \MybbStuff_MyAlerts_AlertTypeManager::getInstance();

            // location installation handling
            if (($location = $mybb->get_input('dvz_me_alerts_install')) && verify_post_check($mybb->get_input('my_post_key'))) {
                if (in_array($location, self::$locations)) {
                    if (!$alertTypeManager->getByCode('dvz_mentions_' . $location)) {
                        self::installLocation($location);
                        flash_message($lang->dvz_me_alerts_installed, 'success');
                        admin_redirect('index.php?module=config-plugins');
                    }
                }
            }

            // location uninstallation handling
            if (($location = $mybb->get_input('dvz_me_alerts_uninstall')) && verify_post_check($mybb->get_input('my_post_key'))) {
                if (in_array($location, self::$locations)) {
                    if ($alertTypeManager->getByCode('dvz_mentions_' . $location)) {
                        self::uninstallLocation($location);
                        flash_message($lang->dvz_me_alerts_uninstalled, 'success');
                        admin_redirect('index.php?module=config-plugins');
                    }
                }
            }

            // location list
            $appendix = $lang->dvz_me_alerts;

            $locationList = [];

            foreach (self::$locations as $location) {

                $installed = $alertTypeManager->getByCode('dvz_mentions_' . $location);

                $locationName = $installed ? '<b>' . $location . '</b>' : $location;
                $actionUrl = 'index.php?module=config-plugins&amp;dvz_me_alerts_' . ($installed ? 'uninstall' : 'install') . '=' . htmlspecialchars_uni($location) . '&amp;my_post_key=' . $mybb->post_code;
                $actionText = $installed ? $lang->dvz_me_alerts_uninstall : $lang->dvz_me_alerts_install;

                $locationList[] = $locationName . ' <a href="' . $actionUrl . '">[' . $actionText . ']</a>';

            }

            $appendix .= ' ' . implode(' &middot; ', $locationList) . '<br>';

            Config::$descriptionAppendix .= $appendix;

        }
    }

    static function installLocation($location)
    {
        global $db, $cache;

        $alertTypeManager = \MybbStuff_MyAlerts_AlertTypeManager::getInstance();

        $alertType = new \MybbStuff_MyAlerts_Entity_AlertType();
        $alertType->setCode('dvz_mentions_' . $location);

        $alertTypeManager->add($alertType);
    }

    static function uninstallLocation($location)
    {
        global $db, $cache;

        $alertTypeManager = \MybbStuff_MyAlerts_AlertTypeManager::getInstance();

        $alertTypeManager->deleteByCode('dvz_mentions_' . $location);
    }

    static function queueAlerts($location, $alertDetails, $locationData, $calledUids, $selfUid = false)
    {

        if ($selfUid) {
            // remove self calls
            if (($index = array_search($selfUid, $calledUids)) !== false) {
                unset($calledUids[ $index ]);
            }
        }

        $alertType = \MybbStuff_MyAlerts_AlertTypeManager::getInstance()->getByCode('dvz_mentions_' . $location);

        $alerts = [];

        foreach ($calledUids as $uid) {

            if (call_user_func_array(['DVZMentions\\' . ucfirst($location) . 'Alerts', 'alertPossible'], [$locationData, $uid])) {

                if ($alertType && $alertType->getEnabled()) {

                    $alert = new \MybbStuff_MyAlerts_Entity_Alert();

                    $alert
                        ->setType($alertType)
                        ->setUserId($uid)
                        ->setExtraDetails($alertDetails)
                    ;

                    if (isset($locationData['author']) && $locationData['author']) {
                        $alert->setFromUserId($locationData['author']);
                    }

                    $alerts[] = $alert;

                }

            }

        }

        if ($alerts) {
            \MybbStuff_MyAlerts_AlertManager::getInstance()->addAlerts($alerts);
        }
    }

    static function myalertsInit()
    {
        defined('MYBBSTUFF_CORE_PATH') or define('MYBBSTUFF_CORE_PATH', MYBB_ROOT . 'inc/plugins/MybbStuff/Core/');
        defined('MYALERTS_PLUGIN_PATH') or define('MYALERTS_PLUGIN_PATH', MYBB_ROOT . 'inc/plugins/MybbStuff/MyAlerts');
        defined('PLUGINLIBRARY') or define('PLUGINLIBRARY', MYBB_ROOT . 'inc/plugins/pluginlibrary.php');

        require_once MYBBSTUFF_CORE_PATH . 'ClassLoader.php';

        $classLoader = new \MybbStuff_Core_ClassLoader();
        $classLoader->registerNamespace('MybbStuff_MyAlerts', [MYALERTS_PLUGIN_PATH . '/src']);
        $classLoader->register();
    }

    static function myalertsIntegrable()
    {
        global $cache;

        $pluginsCache = $cache->read('plugins');

        if (in_array('myalerts', $pluginsCache['active'])) {
            if ($euantor_plugins = $cache->read('euantor_plugins')) {
                if (isset($euantor_plugins['myalerts']['version'])) {
                    $version = explode('.', $euantor_plugins['myalerts']['version']);
                    if ($version[0] == '2' && $version[1] == '0') {
                        return true;
                    }
                }
            }
        }

        return false;
    }
}
