<?php
/* by Tomasz 'Devilshakerz' Mlynski [devilshakerz.com]; Copyright (C) 2014-2016
 released under Creative Commons BY-NC-SA 4.0 license: https://creativecommons.org/licenses/by-nc-sa/4.0/ */

namespace DVZMentions;

class PostAlerts
{
    static $oldMessage;

    static function newThread()
    {
        global $mybb, $new_thread, $uid, $thread_info;

        $message = $new_thread['message'];

        $alertDetails = [
            'tid'     => (int)$thread_info['tid'],
            'pid'     => (int)$thread_info['pid'],
            'subject' => $new_thread['subject'],
        ];

        $locationData = [
            'fid'     => (int)$new_thread['fid'],
            'tid'     => (int)$thread_info['tid'],
            'pid'     => (int)$thread_info,
            'author'  => $uid,
            'subject' => $new_thread['subject'],
        ];

        $calledUids = Parser::getCalledUids($message);

        if ($calledUids) {
            Alerts::QueueAlerts('post', $alertDetails, $locationData, $calledUids, $mybb->user['uid']);
        }
    }

    static function newReply()
    {
        global $post, $pid, $thread, $thread_subject;

        $message = $post['message'];

        $alertDetails = [
            'tid'     => (int)$thread['tid'],
            'pid'     => $pid,
            'subject' => $thread_subject,
        ];

        $locationData = [
            'fid'     => (int)$thread['fid'],
            'tid'     => (int)$thread['tid'],
            'pid'     => (int)$pid,
            'author'  => $post['uid'],
            'subject' => $thread_subject,
        ];

        $calledUids = Parser::getCalledUids($message);

        if ($calledUids) {
            Alerts::queueAlerts('post', $alertDetails, $locationData, $calledUids, $post['uid']);
        }
    }

    static function editpostStart()
    {
        global $post;
        self::$oldMessage = $post['message'];
    }

    static function postUpdate(\PostDataHandler $PostDataHandler)
    {
        global $post, $thread;

        if (self::$oldMessage && isset($PostDataHandler->data['message'])) {

            $postData = $post;

            $message = $PostDataHandler->data['message'];

            $alertDetails = [
                'tid'     => (int)$thread['tid'],
                'pid'     => (int)$postData['pid'],
                'subject' => $thread['subject'],
            ];

            $locationData = [
                'fid'    => (int)$thread['fid'],
                'tid'    => (int)$thread['tid'],
                'pid'    => (int)$postData['pid'],
                'author' => (int)$postData['uid'],
            ];

            $calledUidsOld = Parser::getCalledUids(self::$oldMessage);
            $calledUids = Parser::getCalledUids($message);

            $calledUids = array_diff($calledUids, $calledUidsOld);

            if ($calledUids) {
                Alerts::queueAlerts('post', $alertDetails, $locationData, $calledUids, (int)$postData['uid']);
            }

        }
    }

    static function alertPossible($locationData, $uid)
    {
        global $db, $cache;

        static $userCache;

        // user info
        if (!isset($userCache[ $uid ])) {

            $userDataRaw = $db->fetch_array(
                $db->simple_select('users', 'usergroup,additionalgroups,ignorelist', 'uid=' . (int)$uid)
            );

            if ($userDataRaw) {
                $userCache[ $uid ] = [
                    'ignored_users' => explode(',', $userDataRaw['ignorelist']),
                    'groups'        => array_merge([$userDataRaw['usergroup']], explode(',', $userDataRaw['additionalgroups'])),
                ];
            } else {
                return false;
            }
        }

        $userData = $userCache[ $uid ];

        // ignore list
        if (in_array($locationData['author'], $userData['ignored_users'])) {
            return false;
        }

        $forumData = get_forum($locationData['fid'], true);

        // forum password
        if ($forumData['password']) {
            return false;
        }

        $threadData = get_thread($locationData['tid']);
        $postData = get_post($locationData['pid']);

        // forum permissions
        $forumPermissions = $cache->read('forumpermissions')[ $locationData['fid'] ];

        $permissions = [
            'canview'               => false,
            'canviewthreads'        => false,
            'canonlyviewownthreads' => null, // any "false" supersedes "true"
        ];

        foreach ($userData['groups'] as $gid) {

            if (isset($forumPermissions[ $gid ])) {
                // get forum-specific group permissions
                $groupPermissions = $forumPermissions[ $gid ];
            } else {
                // default to global group permissions
                $groupPermissions = $cache->read('usergroups')[ $gid ];
            }

            if ($groupPermissions['canview']) {
                $permissions['canview'] = true;
            }

            if ($groupPermissions['canviewthreads']) {
                $permissions['canviewthreads'] = true;
            }

            if ($groupPermissions['canonlyviewownthreads']) {
                if ($permissions['canonlyviewownthreads'] == null) {
                    $permissions['canonlyviewownthreads'] = true;
                }
            } else {
                $permissions['canonlyviewownthreads'] = false;
            }

        }

        if (!$permissions['canview'] || !$permissions['canviewthreads']) {
            return false;
        }

        if ($permissions['canonlyviewownthreads']) {

            if ($threadData['uid'] != $uid) {
                return false;
            }

        }

        // thread permissions
        if ($threadData['visible'] != 1) {

            if (
                $threadData['visible'] == 0 && !is_moderator($locationData['fid'], 'canviewunapprove', $uid) ||
                $threadData['visible'] == -1 && !is_moderator($locationData['fid'], 'canviewdeleted', $uid)
            ) {
                return false;
            }

        }

        // post permissions
        if ($postData['visible'] != 1) {

            if (
                $postData['visible'] == 0 && !is_moderator($locationData['fid'], 'canviewunapprove', $uid) ||
                $postData['visible'] == -1 && !is_moderator($locationData['fid'], 'canviewdeleted', $uid)
            ) {
                return false;
            }

        }

        return true;
    }

    static function myalertsLoadLang()
    {
        global $lang;
        $lang->load('dvz_mentions_alerts_post');
    }
}
