<?php
/* by Tomasz 'Devilshakerz' Mlynski [devilshakerz.com]; Copyright (C) 2014-2016
 released under Creative Commons BY-NC-SA 4.0 license: https://creativecommons.org/licenses/by-nc-sa/4.0/ */

namespace DVZMentions;

class Parser
{
    static function getCalls($message)
    {
        $lengthRange = Config::$minLength . ',' . Config::$maxLength;

        preg_match_all('/(?:^|[^\w])@(?:(("|\'|`)[^\n<>,;&\\\]{' . $lengthRange . '}?\2)|([^\n<>,;&\\\"\'`\.:\-+=~@#$%^*!?()\[\]{}\s]{' . $lengthRange . '}))/u', $message, $regexMatches, PREG_OFFSET_CAPTURE);

        $matches = [];

        // validate & merge regex catch groups, build usernames index
        if ($regexMatches) {

            foreach ($regexMatches[1] as $match) {
                if (!empty($match[0])) {

                    $username = substr($match[0], 1, strlen($match[0]) - 2);

                    $matches[] = [
                        'offset'   => $match[1],
                        'username' => $username,
                        'quotes'   => $match[0][0],
                    ];

                }
            }

            foreach ($regexMatches[3] as $match) {
                if (!empty($match[0]) && !in_array($match[0], Config::$ignore)) {

                    $matches[] = [
                        'offset'   => $match[1],
                        'username' => $match[0],
                        'quotes'   => false,
                    ];

                }
            }

        }

        // mention limit
        if (Config::$mentionLimit && count($matches) > Config::$mentionLimit) {
            return [];
        }

        // sort by match offset
        usort($matches, function ($a, $b) {
            return version_compare($a['offset'], $b['offset']);
        });

        return $matches;
    }

    static function getUsernamesFromMatchSet($matches, $usernames = [])
    {
        foreach ($matches as $match) {

            if (!in_array(mb_strtolower($match['username']), $usernames)) {
                $usernames[] = mb_strtolower($match['username']);
            }

        }

        return $usernames;
    }

    static function getUsersByUsername($usernames, $data = 'uid')
    {
        global $db;

        $users = [];

        if ($usernames) {

            // escape values
            array_walk($usernames, function (&$username) use ($db) {
                $username = "'" . $db->escape_string($username) . "'";
            });

            $data = $db->simple_select('users', $data . ',username', (Config::$csCollation ? 'LOWER(username)' : 'username') . ' IN (' . implode(',', $usernames) . ')');

            while ($row = $db->fetch_array($data)) {
                $users[ mb_strtolower($row['username']) ] = $row;
            }

        }

        return $users;
    }

    static function getCalledUids($message, $stripQuotes = true)
    {
        if ($stripQuotes) {
            $message = preg_replace('/\[(quote|code|php)(=[^\]]*)?\](.*?)\[\/\1\]/si', null, $message);
        }

        $users = self::getUsersByUsername(
            self::getUsernamesFromMatchSet(
                self::getCalls($message)
            )
        );

        $uids = [];

        foreach ($users as $user) {
            $uids[] = $user['uid'];
        }

        return array_unique($uids);
    }
}
