<?php

if(!defined("IN_MYBB"))
{
	die("Direct initialization of this file is not allowed.<br /><br />Please make sure IN_MYBB is defined.");
}

$plugins->add_hook("global_end", "newpoints_buypageaccess_access");

function newpoints_buypageaccess_info()
{
	return array(
		"name"			=> "Buy Page Access",
		"description"	=> "Allows users to buy access to certain pages of your site.",
		"website"		=> "",
		"author"		=> "chack1172",
		"authorsite"	=> "chack1172.altervista.org",
		"version"		=> "1.0",
		"guid" 			=> "",
		"compatibility" => "*"
	);
}

function newpoints_buypageaccess_install()
{
	global $db;
	
	// add settings
	newpoints_add_setting('newpoints_buypageaccess_pages', 'newpoints_buypageaccess', 'Pages', 'Pages which are affected by this plugin (seperated by commas: vip.php,vip2.php). (leave blank if you don\'t want any pages to be affected)', 'text', '', 1);
	newpoints_add_setting('newpoints_buypageaccess_time', 'newpoints_buypageaccess', 'Amount of seconds', 'Enter the amount of seconds users will have access to the pages they buy. Set to 0 if unlimited. (default is one hour: 3600)', 'numeric', '3600', 2);
	newpoints_add_setting('newpoints_buypageaccess_fee', 'newpoints_buypageaccess', 'Fee', 'Amount of money users need to pay to access the pages.', 'text', '100', 3);

	$db->write_query("CREATE TABLE `".TABLE_PREFIX."newpoints_buypageaccess` (
	  `pid` int(10) UNSIGNED NOT NULL auto_increment,
	  `page` varchar(255) NOT NULL,
	  `uid` bigint(30) unsigned NOT NULL default '0',
	  `date` bigint(30) unsigned NOT NULL default '0',
	  PRIMARY KEY  (`pid`)
		) ENGINE=MyISAM");
	
	rebuild_settings();
}

function newpoints_buypageaccess_is_installed()
{
	global $db;
	if($db->table_exists('newpoints_buypageaccess'))
        return true;
    else
        return false;
}

function newpoints_buypageaccess_uninstall()
{
	global $db;

	// delete settings
	newpoints_remove_settings("'newpoints_buypageaccess_pages','newpoints_buypageaccess_time','newpoints_buypageaccess_fee'");
	rebuild_settings();
	
	if($db->table_exists('newpoints_buypageaccess'))
		$db->drop_table('newpoints_buypageaccess');
	
	newpoints_remove_log(array('buypageaccess'));
}

function newpoints_buypageaccess_activate()
{
    
}

function newpoints_buypageaccess_deactivate()
{
    
}

function newpoints_buypageaccess_access()
{
	global $mybb, $db, $lang;
	
    $page = get_current_location();
    $page = my_substr($page, -my_strpos(strrev($page), "/"));
    $temp = explode("?", $page);
    $page = $temp[0];
    
    if(empty($mybb->settings['newpoints_buypageaccess_pages']))
        return;
    
	$affected_pages = explode(',', $mybb->settings['newpoints_buypageaccess_pages']);
	if (!in_array($page, $affected_pages))
		return;
		
	// If we're a guest, we cant' browse this forum because we can't pay for it
	if (!$mybb->user['uid'])
		error_no_permission();
	
	newpoints_lang_load('newpoints_buypageaccess');
	
    $time_check = '';
	if ($mybb->settings['newpoints_buypageaccess_time'] > 0) {
        $time = TIME_NOW - intval($mybb->settings['newpoints_buypageaccess_time']);
		$time_check = 'AND `date` > ('.$time.')';
    }
	
	// check if we've bought access to this forum
    $page = $db->escape_string($page);
	$query = $db->simple_select('newpoints_buypageaccess', '*', '`uid`='.$mybb->user['uid'].' AND `page`=\''.$page.'\' '.$time_check);
	$access = $db->fetch_array($query);
	if (!$access)
	{
		// we need to pay
		
		if ($mybb->request_method == "post" && $mybb->input['action'] == 'do_pay')
		{
			// insert data into database
			$insert_array = array(
				'page' => $page,
				'uid' => intval($mybb->user['uid']),
				'date' => TIME_NOW
			);
			$db->insert_query('newpoints_buypageaccess', $insert_array);
			
			// get money from user
			newpoints_addpoints($mybb->user['uid'], -floatval($mybb->settings['newpoints_buypageaccess_fee']));
			
			// log
			newpoints_log('buypageaccess', $lang->sprintf($lang->newpoints_buypageaccess_log, $page, $mybb->settings['newpoints_buypageaccess_fee']));
		}
		else {
			// Show page telling the user to pay
			
			// do we have enough points?
			if (floatval($mybb->user['newpoints']) < floatval($mybb->settings['newpoints_buypageaccess_fee']))
			{
				error($lang->sprintf($lang->newpoints_buypageaccess_no_points, newpoints_format_points(floatval($mybb->settings['newpoints_buypageaccess_fee']))));
			}
			else {
				if (!$time_check)
					$time_check = $lang->newpoints_buypageaccess_unlimited;
				else
				{
					$secondsaccess = $mybb->settings['newpoints_buypageaccess_time'];
					$hoursaccess = floor($secondsaccess / 3600);
					$minsaccess = floor(($secondsaccess / 60) % 60);
					$secsaccess = floor($secondsaccess % 60);
					
					$time_check = $hoursaccess."h ".$minsaccess."m ".$secsaccess."s";
				}
			
				$form = $lang->sprintf($lang->newpoints_buypageaccess_notice, newpoints_format_points(floatval($mybb->settings['newpoints_buypageaccess_fee'])), $time_check)."<br /><form action=\"{$page}\" method=\"post\">";
			
				$form .= "<input type=\"hidden\" name=\"action\" value=\"do_pay\">\n";
				
				// Make sure we escape these before outputting them
				$input = array (
					'datecut' => intval($mybb->input['datecut']),
					'order' => htmlspecialchars($mybb->input['order']),
					'sortby' => htmlspecialchars($mybb->input['sortby']),
					'page' => intval($mybb->input['page']),
				);
				
				foreach ($input as $name => $value)
				{
					if ($value == 0 || $value == '')
						continue;
					
					$form .= "<input type=\"hidden\" name=\"".$name."\" value=\"".$value."\">\n";
				}
			
				$form .= "<input type=\"submit\" value=\"".$lang->newpoints_buypageaccess_confirm."\">\n";
				
				$form .= "</form>";
			
				error($form, $lang->newpoints_buypageaccess_verify_payment);
			}
		}
	}
}