<?php
/***************************************************************************
 *
 *   Virtual Stock Market plugin (/inc/plugins/newpoints/languages/english/virtualstock.php)
 *	 Author: ZLight Software
 *   Copyright:  2013 ZLight Software
 *   
 *   Website: http://zlightsoftware.com
 *   License: licence.txt
 *
 *   Integrates a virtual stock market with NewPoints.
 *
 ***************************************************************************/
 
/****************************************************************************
* LICENCE: The licence file is named licence.txt and comes with this product.
* In case licence.txt does not come with this product, please contact 
* ZLight Software immediatly.
****************************************************************************/

// Disallow direct access to this file for security reasons
if(!defined("IN_MYBB"))
{
	die("Direct initialization of this file is not allowed.<br /><br />Please make sure IN_MYBB is defined.");
}

$plugins->add_hook("newpoints_start", "virtualstock_page");

if (defined("IN_ADMINCP"))
{
	$plugins->add_hook('newpoints_admin_load', 'virtualstock_admin');
	$plugins->add_hook('newpoints_admin_newpoints_menu', 'virtualstock_admin_newpoints_menu');
	$plugins->add_hook('newpoints_admin_newpoints_action_handler', 'virtualstock_admin_newpoints_action_handler');
	$plugins->add_hook('newpoints_admin_newpoints_permissions', 'virtualstock_admin_permissions');
}
else
{
	$plugins->add_hook("newpoints_default_menu", "virtualstock_menu");
}

// backup virtualstock fields too
$plugins->add_hook("newpoints_task_backup_tables", "virtualstock_backup");

function virtualstock_info()
{
	return array(
		"name"			=> "Virtual Stock Market",
		"description"	=> "Integrates a virtual stock market with NewPoints.",
		"website"		=> "http://www.zlightsoftware.com",
		"author"		=> "ZLight Software",
		"authorsite"	=> "http://www.zlightsoftware.com",
		"version"		=> "1.0",
		"guid" 			=> "",
		"compatibility" => "2*"
	);
}

function virtualstock_install()
{
	global $db;
	$db->write_query("ALTER TABLE `".TABLE_PREFIX."users` ADD `virtualstock` TEXT NOT NULL;");
	
	$db->write_query("CREATE TABLE `".TABLE_PREFIX."virtualstock_categories` (
	  `cid` bigint(30) UNSIGNED NOT NULL auto_increment,
	  `name` varchar(100) NOT NULL default '',
	  `description` text NOT NULL,
	  `visible` smallint(1) NOT NULL default '1',
	  `icon` varchar(255) NOT NULL default '',
	  `usergroups` varchar(100) NOT NULL default '',
	  `disporder` int(5) NOT NULL default '0',
	  `companies` int(10) NOT NULL default '0',
	  `expanded` smallint(1) NOT NULL default '1',
	  PRIMARY KEY  (`cid`)
		) ENGINE=MyISAM");
	
	$db->write_query("CREATE TABLE `".TABLE_PREFIX."virtualstock_companies` (
	  `iid` bigint(30) UNSIGNED NOT NULL auto_increment,
	  `name` varchar(100) NOT NULL default '',
	  `description` text NOT NULL,
	  `price` DECIMAL(16,2) NOT NULL default '0',
	  `icon` varchar(255) NOT NULL default '',
	  `visible` smallint(1) NOT NULL default '1',
	  `disporder` int(5) NOT NULL default '0',
	  `total` int(10) NOT NULL default '0',
	  `available` int(10) NOT NULL default '0',
	  `minprice` DECIMAL(16,2) NOT NULL default '0',
	  `maxprice` DECIMAL(16,2) NOT NULL default '0',
	  `cid` int(10) NOT NULL default '0',
	  PRIMARY KEY  (`iid`)
		) ENGINE=MyISAM");
		
	$db->write_query("CREATE TABLE `".TABLE_PREFIX."virtualstock_stocks` (
	  `uid` int(10) NOT NULL default '0',
	  `amount` int(10) NOT NULL,
	  `points` decimal(16,2) NOT NULL,
	  `type` tinyint(1) NOT NULL default '0',
	  `date` int(10) NOT NULL,
	  `iid` int(10) NOT NULL default '0'
		) ENGINE=MyISAM");
	
	// create task
	$new_task = array(
		"title" => "Virtual Stock",
		"description" => "Calculates the stock prices.",
		"file" => "virtualstock",
		"minute" => '0',
		"hour" => '*',
		"day" => '*',
		"month" => '*',
		"weekday" => '*',
		"enabled" => '0',
		"logging" => '1'
	);
	
	$new_task['nextrun'] = 0; // once the task is enabled, it will generate a nextrun date
	$tid = $db->insert_query("tasks", $new_task);
}

function virtualstock_is_installed()
{
	global $db;
	if($db->field_exists('virtualstock', 'users'))
	{
		return true;
	}
	return false;
}

function virtualstock_uninstall()
{
	global $db;
	$db->write_query("ALTER TABLE `".TABLE_PREFIX."users` DROP `virtualstock`;");
	
	if($db->table_exists('virtualstock_categories'))
	{
		$db->drop_table('virtualstock_categories');
	}
	
	if($db->table_exists('virtualstock_companies'))
	{
		$db->drop_table('virtualstock_companies');
	}
	
	if($db->table_exists('virtualstock_stocks'))
	{
		$db->drop_table('virtualstock_stocks');
	}
	
	newpoints_remove_log(array('virtualstock_purchase', 'virtualstock_sell'));
	
	$db->delete_query('tasks', 'file=\'virtualstock\'');
}

function virtualstock_activate()
{
	global $db, $mybb;
	
	newpoints_add_template('virtualstock', '
<html>
<head>
<title>{$lang->newpoints} - {$lang->virtualstock}</title>
{$headerinclude}
</head>
<body>
	{$header}
	<table width="100%" border="0" align="center">
		<tr>
			<td valign="top" width="180">
				<table border="0" cellspacing="{$theme[\'borderwidth\']}" cellpadding="{$theme[\'tablespace\']}" class="tborder">
				<tr>
				<td class="thead"><strong>{$lang->newpoints_menu}</strong></td>
				</tr>
				{$options}
				</table>
			</td>
			<td valign="top">
				{$inline_errors}
				<table border="0" cellspacing="{$theme[\'borderwidth\']}" cellpadding="{$theme[\'tablespace\']}" class="tborder">
					<tr>
					<td class="thead" colspan="5"><div style="float: right"><strong><a href="{$mybb->settings[\'bburl\']}/newpoints.php?action=virtualstock&amp;virtualstock_action=mystocks" style="text-decoration: none">{$lang->virtualstock_mystocks}</a></strong></div><strong>{$lang->virtualstock}</strong></td>
					</tr>
					<tr>
					<td class="tcat" width="1%"><strong>{$lang->virtualstock_icon}</strong></td>
					<td class="tcat"><strong>{$lang->virtualstock_name}</strong></td>
					<td class="tcat" width="15%" align="center"><strong>{$lang->virtualstock_price}</strong></td>
					<td class="tcat" width="20%" align="center"><strong>{$lang->virtualstock_stock}</strong></td>
					<td class="tcat" width="15%" align="center"><strong>{$lang->virtualstock_buy}</strong></td>
					</tr>
				</table><br />
				{$cats}
				{$stats}
			</td>
		</tr>
	</table>
	{$footer}
</body>
</html>');
	
	newpoints_add_template('virtualstock_category', '
<table border="0" cellspacing="{$theme[\'borderwidth\']}" cellpadding="{$theme[\'tablespace\']}" class="tborder">
	<tr>
		<td class="thead" colspan="5">
			<div class="expcolimage"><img src="{$theme[\'imgdir\']}/{$expcolimage}" id="cat_{$category[\'cid\']}_img" class="expander" alt="{$expaltext}" title="{$expaltext}" /></div>
			{$category[\'icon\']} <strong>{$category[\'name\']}</strong> <span class="smalltext">{$category[\'description\']}</span>
		</td>
	</tr>


	
		
			{$companies}
		
	</table>
<br />');
	
	newpoints_add_template('virtualstock_company', '
<tr>
	<td class="{$bgcolor}" width="1%" valign="middle" align="center">{$company[\'icon\']}</td>
	<td class="{$bgcolor}" valign="middle">{$company[\'name\']}<br /><span class="smalltext">{$company[\'description\']}</span></td>
	<td class="{$bgcolor}" width="15%" align="center" valign="middle">{$company[\'price\']}</td>
	<td class="{$bgcolor}" width="20%" align="center" valign="middle">{$company[\'available\']} / {$company[\'total\']}</td>
	<td class="{$bgcolor}" width="15%" align="center" valign="middle"><form action="newpoints.php?action=do_virtualstock&amp;virtualstock_action=buy" method="POST"><input type="hidden" name="postcode" value="{$mybb->post_code}" /><input type="hidden" name="iid" value="{$company[\'iid\']}" /><input type="text" name="stocks" class="textbox" /><br /><input type="submit" name="buy" value="{$lang->virtualstock_buy}" /></form></td>
</tr>');
	
	newpoints_add_template('virtualstock_no_companies', '
<tr>
<td class="trow1" colspan="5">{$lang->virtualstock_no_companies}</td>
</tr>');
	
	newpoints_add_template('virtualstock_no_cats', '
<tr>
<td class="trow1" colspan="5">{$lang->virtualstock_no_cats}</td>
</tr>');
	
	newpoints_add_template('virtualstock_mystocks', '
<html>
<head>
<title>{$lang->newpoints} - {$lang->virtualstock_mystocks}</title>
{$headerinclude}
</head>
<body>
{$header}
<table width="100%" border="0" align="center">
<tr>
<td valign="top" width="180">
<table border="0" cellspacing="{$theme[\'borderwidth\']}" cellpadding="{$theme[\'tablespace\']}" class="tborder">
<tr>
<td class="thead"><strong>{$lang->newpoints_menu}</strong></td>
</tr>
{$options}
</table>
</td>
<td valign="top">
{$inline_errors}
<table border="0" cellspacing="{$theme[\'borderwidth\']}" cellpadding="{$theme[\'tablespace\']}" class="tborder">
<tr>
<td class="thead" colspan="4"><strong><div style="float: right"><a href="{$mybb->settings[\'bburl\']}/newpoints.php?action=virtualstock">{$lang->virtualstock}</a></div>{$lang->virtualstock_mystocks}</strong></td>
</tr>
{$companies}
</table>
{$multipage}
</td>
</tr>
</table>
{$footer}
</body>
</html>');
	
	newpoints_add_template('virtualstock_mystocks_company', '
<td class="{$bgcolor}" width="50%" valign="middle" align="center">
<fieldset class="{$invert_bgcolor}" style="width: 80%;"><legend>{$company[\'icon\']} <strong><a href="{$mybb->settings[\'bburl\']}/newpoints.php?action=virtualstock&amp;virtualstock_action=view&amp;iid={$company[\'iid\']}">{$company[\'name\']}</a></strong></legend>
<div align="left"><span class="smalltext">{$company[\'description\']}</span></div>
<div align="left">
{$lang->virtualstock_price}: {$company[\'price\']}<br />
{$lang->virtualstock_stocks}: {$company[\'stocks\']}
</div>
</fieldset>
<fieldset class="{$invert_bgcolor}" style="width: 80%; margin-bottom: 5px;">
<legend>{$lang->virtualstock_options}</legend>
<table border="0">
<tr>
<form action="newpoints.php" method="POST"><input type="hidden" name="action" value="do_virtualstock"><input type="hidden" name="virtualstock_action" value="sell"><input type="hidden" name="iid" value="{$company[\'iid\']}"><input type="hidden" name="postcode" value="{$mybb->post_code}"><input type="text" class="textbox" name="stocks" /><br /><input type="submit" name="submit" value="{$lang->virtualstock_sell}"></form>
</tr>
</table>
</fieldset>
</td>
');
	
	newpoints_add_template('virtualstock_mystocks_company_empty', '
<td class="{$bgcolor}" width="50%" valign="middle"></td>
');
	
	newpoints_add_template('virtualstock_mystocks_no_companies', '
<tr>
<td class="trow1" colspan="2">{$lang->virtualstock_no_companies}</td>
</tr>');
	
	newpoints_add_template('virtualstock_do_action', '
<head>
<title>{$lang->newpoints} - {$lang->virtualstock_action}</title>
{$headerinclude}
</head>
<body>
{$header}
<form action="newpoints.php?action=do_virtualstock" method="POST">
<input type="hidden" name="postcode" value="{$mybb->post_code}">
<input type="hidden" name="virtualstock_action" value="{$virtualstock_action}">
{$fields}
<table border="0" cellspacing="{$theme[\'borderwidth\']}" cellpadding="{$theme[\'tablespace\']}" class="tborder">
<tr>
<td class="thead" colspan="{$colspan}"><strong>{$lang->virtualstock_action}:</strong> {$company[\'name\']}</td>
</tr>
<tr>
{$data}
</tr>
<tr>
<td class="tfoot" width="100%" align="center" colspan="{$colspan}"><input type="submit" name="submit" value="{$lang->virtualstock_confirm}"></td>
</tr>
</table>
</form>
{$footer}
</body>
</html>');
	
	newpoints_add_template('virtualstock_stats', '

<table border="0" cellspacing="{$theme[\'borderwidth\']}" cellpadding="{$theme[\'tablespace\']}" class="tborder">
	<tr>
		<td class="thead" colspan="2"><strong>{$lang->virtualstock_stats}</strong></td>
	</tr>
	<tr>
		<td class="trow1" width="50%"><strong>{$lang->virtualstock_stocks_purchased}:</strong></td>
		<td class="trow1" width="50%">{$totalbought}</td>
	</tr>
	<tr>
		<td class="trow2" width="50%"><strong>{$lang->virtualstock_stocks_sold}:</strong></td>
		<td class="trow2" width="50%">{$totalsold}</td>
	</tr>
	<tr>
		<td class="trow1" width="50%"><strong>{$lang->virtualstock_stocks_current}:</strong></td>
		<td class="trow1" width="50%">{$currentstocks}</td>
	</tr>
	<tr>
		<td class="trow2" width="50%"><strong>{$lang->virtualstock_totalspent}:</strong></td>
		<td class="trow2" width="50%">{$totalspent}</td>
	</tr>
	<tr>
		<td class="trow1" width="50%"><strong>{$lang->virtualstock_totalgained}:</strong></td>
		<td class="trow1" width="50%">{$totalgained}</td>
	</tr>
	<tr>
		<td class="trow2" width="50%"><strong>{$lang->virtualstock_profit}:</strong></td>
		<td class="trow2" width="50%">{$profit}</td>
	</tr>
	<tr>
		<td class="trow1" width="50%"><strong>{$lang->virtualstock_balance}:</strong></td>
		<td class="trow1" width="50%">{$balance}</td>
	</tr>
</table>
<br />');
}

function virtualstock_deactivate()
{
	global $db, $mybb;
	
	newpoints_remove_templates("'virtualstock','virtualstock_category','virtualstock_company','virtualstock_no_companies','virtualstock_no_cats','virtualstock_mystocks','virtualstock_mystocks_company','virtualstock_mystocks_no_items','virtualstock_do_action','virtualstock_stats','virtualstock_mystocks_company_empty'");
}

// show virtual stock in the list
function virtualstock_menu(&$menu)
{
	global $mybb, $lang;
	newpoints_lang_load("virtualstock");
	
	if ($mybb->input['action'] == 'virtualstock')
		$menu[] = "&raquo; <a href=\"{$mybb->settings['bburl']}/newpoints.php?action=virtualstock\">".$lang->virtualstock."</a>";
	else
		$menu[] = "<a href=\"{$mybb->settings['bburl']}/newpoints.php?action=virtualstock\">".$lang->virtualstock."</a>";
}

function virtualstock_get_company($iid = 0)
{
	global $db, $mybb;
	if (!$iid)
		return false;
		
	$query = $db->simple_select('virtualstock_companies', '*', 'iid=\''.intval($iid).'\'');
	$company = $db->fetch_array($query);
		
	return $company;
}

function virtualstock_get_category($cid = 0)
{
	global $db, $mybb;
	if (!$cid)
		return false;
		
	$query = $db->simple_select('virtualstock_categories', '*', 'cid=\''.intval($cid).'\'');
	$cat = $db->fetch_array($query);
		
	return $cat;
}

/* Checks if the primary or any of the additional groups of the current user are in the groups list passed as a parameter
 * @param String group ids seperated by a comma
 * @return true if the user has permissions, false if not
*/
function virtualstock_check_permissions($groups_comma)
{
    global $mybb;
    
    if ($groups_comma == '')
        return false;
    
    $groups = explode(",", $groups_comma);
    $add_groups = explode(",", $mybb->user['additionalgroups']);
    
    if (!in_array($mybb->user['usergroup'], $groups)) { // primary user group not allowed
        // check additional groups
        if ($add_groups) {
            if (count(array_intersect($add_groups, $groups)) == 0)
                return false;
            else
                return true;
        }
        else 
            return false;
    }
    else
        return true;
}

function virtualstock_page()
{
	global $mybb, $db, $lang, $cache, $theme, $header, $templates, $plugins, $headerinclude, $footer, $options, $inline_errors;
	
	if (!$mybb->user['uid'])
		return;
		
	newpoints_lang_load("virtualstock");
		
	if ($mybb->input['action'] == "do_virtualstock")
	{
		verify_post_check($mybb->input['postcode']);
		
		$plugins->run_hooks("newpoints_do_virtualstock_start");
		
		switch ($mybb->input['virtualstock_action'])
		{
			case 'buy':
				$plugins->run_hooks("virtualstock_buy_start");
			
				// check if the company exists
				if (!($company = virtualstock_get_company($mybb->input['iid'])))
				{
					error($lang->virtualstock_invalid_company);
				}
				
				// check if the company is assigned to category
				if (!($cat = virtualstock_get_category($company['cid'])))
				{
					error($lang->virtualstock_invalid_cat);
				}

				// check if we have permissions to view the parent category
				if (!virtualstock_check_permissions($cat['usergroups']))
				{
					error_no_permission();
				}
				
				$stocks = (int)$mybb->input['stocks'];
				if($stocks <= 0)
				{
					error($lang->virtualstock_invalid_stocks);
				}
				
				if ($company['visible'] == 0 || $cat['visible'] == 0)
					error_no_permission();
					
				if (floatval($stocks*$company['price']) > floatval($mybb->user['newpoints']))
				{
					$errors[] = $lang->virtualstock_not_enough;
				}
				
				if ($stocks > $company['available'])
				{
					$errors[] = $lang->virtualstock_out_of_stock;
				}
				
				if (!empty($errors))
				{
					$inline_errors = inline_error($errors, $lang->virtualstock_inline_errors);
					$mybb->input = array();
					$mybb->input['action'] = 'virtualstock';
				}
				else {
					$mycompanies = @unserialize($mybb->user['virtualstock']);
					if (!$mycompanies)
						$mycompanies = array();
						
					if(!isset($mycompanies[$company['iid']]))
						$mycompanies[$company['iid']] = $stocks;
					else
						$mycompanies[$company['iid']] += $stocks;
					
					$db->update_query('users', array('virtualstock' => serialize($mycompanies)), 'uid=\''.$mybb->user['uid'].'\'');
					
					// update available stocks
					$db->update_query('virtualstock_companies', array('available' => $company['available']-$stocks), 'iid=\''.$company['iid'].'\'');
					
					// get money from user
					newpoints_addpoints($mybb->user['uid'], -(floatval($company['price']*$stocks)));
					
					$plugins->run_hooks("virtualstock_buy_end", $company);
					
					// insert stocks information
					$db->insert_query('virtualstock_stocks', array('uid' => (int)$mybb->user['uid'], 'type' => 0, 'date' => TIME_NOW, 'amount' => $stocks, 'points' => floatval($company['price']*$stocks), 'iid' => $company['iid']));
					
					// log purchase
					newpoints_log('virtualstock_purchase', $lang->sprintf($lang->virtualstock_purchased_log, $company['iid'], $company['price']));
					
					redirect($mybb->settings['bburl']."/newpoints.php?action=virtualstock", $lang->virtualstock_company_bought, $lang->virtualstock_company_bought_title);
				}
			break;
			
			case 'sell':
				$plugins->run_hooks("virtualstock_sell_start");
			
				// check if the company exists
				if (!($company = virtualstock_get_company($mybb->input['iid'])))
				{
					error($lang->virtualstock_invalid_company);
				}
				
				// check if the company is assigned to category
				if (!($cat = virtualstock_get_category($company['cid'])))
				{
					error($lang->virtualstock_invalid_cat);
				}

				// check if we have permissions to view the parent category
				if (!virtualstock_check_permissions($cat['usergroups']))
				{
					error_no_permission();
				}
				
				if ($company['visible'] == 0 || $cat['visible'] == 0)
					error_no_permission();
				
				$mycompanies = @unserialize($mybb->user['virtualstock']);
				if (!$mycompanies)
					error($lang->virtualstock_inventory_empty);
				
				// make sure we own stocks of this company
				if(!isset($mycompanies[$company['iid']]))
					error($lang->virtualstock_selected_company_not_owned);
					
				$stocks = (int)$mybb->input['stocks'];
				if($stocks <= 0 || $stocks > $mycompanies[$company['iid']])
				{
					error($lang->virtualstock_invalid_stocks);
				}
					
				$lang->virtualstock_action = $lang->virtualstock_sell_stocks;
				$company['name'] = htmlspecialchars_uni($company['name']);
				
				global $virtualstock_action, $data, $colspan;
				$colspan = 1;
				$virtualstock_action = 'do_sell';
				$fields = '<input type="hidden" name="iid" value="'.$company['iid'].'" />';
				$fields .= '<input type="hidden" name="stocks" value="'.$stocks.'" />';
				$data = "<td class=\"trow1\" width=\"100%\">".$lang->sprintf($lang->virtualstock_sell_company_confirm, htmlspecialchars_uni($company['name']), $stocks, newpoints_format_points(floatval($company['price'])))."</td>";
				
				$plugins->run_hooks("virtualstock_sell_end");
				
				eval("\$page = \"".$templates->get('virtualstock_do_action')."\";");
				output_page($page);
			break;
			
			case 'do_sell':
				$plugins->run_hooks("virtualstock_do_sell_start");
			
				// check if the company exists
				if (!($company = virtualstock_get_company($mybb->input['iid'])))
				{
					error($lang->virtualstock_invalid_company);
				}
				
				// check if the company is assigned to category
				if (!($cat = virtualstock_get_category($company['cid'])))
				{
					error($lang->virtualstock_invalid_cat);
				}

				// check if we have permissions to view the parent category
				if (!virtualstock_check_permissions($cat['usergroups']))
				{
					error_no_permission();
				}
				
				if ($company['visible'] == 0 || $cat['visible'] == 0)
					error_no_permission();
				
				$mycompanies = @unserialize($mybb->user['virtualstock']);
				if (!$mycompanies)
					error($lang->virtualstock_inventory_empty);
				
				// make sure we own stocks of this company
				if(!isset($mycompanies[$company['iid']]))
					error($lang->virtualstock_selected_company_not_owned);
					
				$stocks = (int)$mybb->input['stocks'];
				if($stocks <= 0 || $stocks > $mycompanies[$company['iid']])
				{
					error($lang->virtualstock_invalid_stocks);
				}
					
				// remove stocks from our inventory
				$mycompanies[$company['iid']] -= $stocks;
				if($mycompanies[$company['iid']] == 0)
				{
					unset($mycompanies[$company['iid']]);
				}
				
				$db->update_query('users', array('virtualstock' => serialize($mycompanies)), 'uid=\''.$mybb->user['uid'].'\'');
				
				// update available stocks
				$db->update_query('virtualstock_companies', array('available' => $company['available']+$stocks), 'iid=\''.$company['iid'].'\'');
				
				newpoints_addpoints($mybb->user['uid'], floatval($company['price']*$stocks));
				
				$plugins->run_hooks("virtualstock_do_sell_end");
				
				// insert stocks information
				$db->insert_query('virtualstock_stocks', array('uid' => (int)$mybb->user['uid'], 'type' => 1, 'date' => TIME_NOW, 'amount' => $stocks, 'points' => floatval($company['price']*$stocks), 'iid' => $company['iid']));
					
				// log
				newpoints_log('virtualstock_sell', $lang->sprintf($lang->virtualstock_sell_log, $company['iid'], $stocks, floatval($company['price'])));
					
				redirect($mybb->settings['bburl']."/newpoints.php?action=virtualstock&amp;virtualstock_action=mystocks", $lang->virtualstock_company_sell, $lang->virtualstock_company_sell_title);
			break;
			
			default:
				error_no_permission();
		}
		
		$plugins->run_hooks("virtualstock_do_end");
	}
	
	// virtual stock page
	if ($mybb->input['action'] == "virtualstock")
	{
		$plugins->run_hooks("virtualstock_start");
		
		if ($mybb->input['virtualstock_action'] == 'mystocks')
		{
			$uid = intval($mybb->input['uid']);
			$uidpart = '';
			if ($uid > 0)
			{
				$user = get_user($uid);
				// we're viewing someone else's inventory
				if (!empty($user))
				{
					// we can't view others inventories if we don't have enough previleges
					if ($mybb->settings['virtualstock_viewothers'] != 1 && $mybb->usergroup['cancp'] != 1 && $mybb->user['uid'] != $uid)
						error_no_permission();
						
					$mycompanies = @unserialize($user['virtualstock']);
					$lang->virtualstock_mystocks = $lang->sprintf($lang->virtualstock_companies_username, htmlspecialchars_uni($user['username']));
					$uidpart = "&amp;uid=".$uid; // we need this for pagination
				}
				else
					$mycompanies = @unserialize($mybb->user['virtualstock']);
			}
			else
				$mycompanies = @unserialize($mybb->user['virtualstock']);
				
			$companies = '';
			$newrow = true;
			$invert_bgcolor = alt_trow();
				
			require_once MYBB_ROOT."inc/class_parser.php";
			$parser = new postParser;
				
			$parser_options = array(
				'allow_mycode' => 1,
				'allow_smilies' => 1,
				'allow_imgcode' => 0,
				'allow_html' => 0,
				'filter_badwords' => 1
			);
			
			if (!empty($mycompanies))
			{
				// pagination
				$per_page = 10;
				$mybb->input['page'] = intval($mybb->input['page']);
				if($mybb->input['page'] && $mybb->input['page'] > 1)
				{
					$mybb->input['page'] = intval($mybb->input['page']);
					$start = ($mybb->input['page']*$per_page)-$per_page;
				}
				else
				{
					$mybb->input['page'] = 1;
					$start = 0;
				}
				
				// total items
				$total_rows = $db->fetch_field($db->simple_select("virtualstock_companies", "COUNT(iid) as companies", 'visible=1 AND iid IN ('.implode(',', array_keys($mycompanies)).')'), "companies");
				
				// multi-page
				if ($total_rows > $per_page)
					$multipage = multipage($total_rows, $per_page, $mybb->input['page'], $mybb->settings['bburl']."/newpoints.php?action=virtualstock&virtualstock_action=mystocks".$uidpart);
				
				$query = $db->simple_select('virtualstock_companies', '*', 'visible=1 AND iid IN ('.implode(',', array_keys($mycompanies)).')', array('limit' => "{$start}, {$per_page}"));
				while ($company = $db->fetch_array($query))
				{
					if ($newrow === true)
					{
						$trstart = '<tr>';
						$trend = '';
						$newrow = false;
					}
					elseif ($newrow === false)
					{
						$trstart = '';
						$trend = '</tr>';
						$newrow = true;
					}
						
					$company['description'] = $parser->parse_message($company['description'], $parser_options);
					
					$company['price'] = newpoints_format_points($company['price']);
					
					// build icon
					if ($company['icon'] != '')
					{
						$company['icon'] = htmlspecialchars_uni($company['icon']);
						$company['icon'] = '<img src="'.$mybb->settings['bburl'].'/'.$company['icon'].'" />';
					}
					else
						$company['icon'] = '<img src="'.$mybb->settings['bburl'].'/images/newpoints/default.png" />';
						
					$company['stocks'] = $mycompanies[$company['iid']];
					
					$bgcolor = alt_trow();
					$invert_bgcolor = alt_trow();
					eval("\$companies .= \"".$trstart.$templates->get('virtualstock_mystocks_company').$trend."\";");
				}
				
				if (!$companies)
				{
					eval("\$companies = \"".$templates->get('virtualstock_mystocks_no_items')."\";");
				}
				else {
					if ($newrow === false) // we haven't closed the row, that means there's a missing td
					{
						eval("\$companies .= \"".$templates->get('virtualstock_mystocks_company_empty')."</tr>"."\";");
						$newrow = true;
					}
				}
			}
			else {
				eval("\$companies = \"".$templates->get('virtualstock_mystocks_no_items')."\";");
			}
			
			eval("\$page = \"".$templates->get('virtualstock_mystocks')."\";");
		}
		else {
			
			global $cats, $companies;
			
			// get categories
			$query = $db->simple_select('virtualstock_categories', '*', '', array('order_by' => 'disporder', 'order_dir' => 'ASC'));
			while ($cat = $db->fetch_array($query))
			{
				$categories[$cat['cid']] = $cat;
			}
			
			// get companies and store them in their categories
			$query = $db->simple_select('virtualstock_companies', '*', 'visible=1 AND cid>0', array('order_by' => 'disporder', 'order_dir' => 'ASC'));
			while ($company = $db->fetch_array($query))
			{
				$companies_array[$company['cid']][$company['iid']] = $company;
			}
			
			$cats = '';
			$bgcolor = '';
			$bgcolor = alt_trow();
			
			// build companies and categories
			if (!empty($categories))
			{
				foreach ($categories as $cid => $category)
				{
					$companies = '';
					
					if ($category['companies'] > 0 && !empty($companies_array[$category['cid']]))
					{
						foreach ($companies_array as $cid => $member)
						{
							if ($cid != $category['cid'])
								continue;
								
							$bgcolor = alt_trow();
							foreach ($member as $iid => $company)
							{
								// skip hidden companies
								if ($company['visible'] == 0)
									continue;
								
								$company['name'] = htmlspecialchars_uni($company['name']);
								$company['description'] = htmlspecialchars_uni($company['description']);
								$company['price'] = newpoints_format_points($company['price']);
								
								// build icon
								if ($company['icon'] != '')
								{
									$company['icon'] = htmlspecialchars_uni($company['icon']);
									$company['icon'] = '<img src="'.$mybb->settings['bburl'].'/'.$company['icon'].'">';
								}
								else
									$company['icon'] = '<img src="'.$mybb->settings['bburl'].'/images/newpoints/default.png">';
								
								eval("\$companies .= \"".$templates->get('virtualstock_company')."\";");
							}
						}
					}
					else
						eval("\$companies = \"".$templates->get('virtualstock_no_companies')."\";");
						
					// if it's not visible, don't show it
					if ($category['visible'] == 0)
						continue;
						
					// check if we have permissions to view the category
					if (!virtualstock_check_permissions($category['usergroups']))
						continue;
						
					// Expanded by default feature
					global $extdisplay, $expcolimage, $expdisplay, $expaltext, $icon;
					
					$expdisplay = '';
					if(intval($category['expanded']) == 0)
					{
						$expcolimage = "collapse_collapsed.gif";
						$expdisplay = "display: none;";
						$expaltext = "[+]";
					}
					else
					{
						$expcolimage = "collapse.gif";
						$expaltext = "[-]";
					}

					// build icon
					if ($category['icon'] != '')
					{
						$category['icon'] = htmlspecialchars_uni($category['icon']);
						$category['icon'] = '<img src="'.$mybb->settings['bburl'].'/'.$category['icon'].'" style="vertical-align:middle">';
					}
					
					// sanitize html
					$category['description'] = htmlspecialchars_uni($category['description']);
					$category['name'] = htmlspecialchars_uni($category['name']);
					
					eval("\$cats .= \"".$templates->get('virtualstock_category')."\";");
				}
			}
			else {
				eval("\$cats = \"".$templates->get('virtualstock_no_cats')."\";");
			}
			
			if($mybb->user['uid'] > 0)
			{
				// Statistics - Total Points spent
				$totalspent = (int)$db->fetch_field($db->simple_select('virtualstock_stocks', 'SUM(points) as totalspent', 'type=0 AND uid='.(int)$mybb->user['uid']), 'totalspent');
				$totalgained = (int)$db->fetch_field($db->simple_select('virtualstock_stocks', 'SUM(points) as totalgained', 'type=1 AND uid='.(int)$mybb->user['uid']), 'totalgained');
				$totalbought = (int)$db->fetch_field($db->simple_select('virtualstock_stocks', 'SUM(amount) as totalbought', 'type=0 AND uid='.(int)$mybb->user['uid']), 'totalbought');
				$totalsold = (int)$db->fetch_field($db->simple_select('virtualstock_stocks', 'SUM(amount) as totalsold', 'type=1 AND uid='.(int)$mybb->user['uid']), 'totalsold');
				
				$profit = newpoints_format_points($totalgained - $totalspent);
				$currentstocks = (int)($totalbought-$totalsold);
				$currentstocks = (int)($totalbought-$totalsold);
				$balance = newpoints_format_points($mybb->user['newpoints']);
				
				eval("\$stats = \"".$templates->get('virtualstock_stats')."\";");
			}
			
			eval("\$page = \"".$templates->get('virtualstock')."\";");
		}
		
		$plugins->run_hooks("virtualstock_end");
		
		// output page
		output_page($page);
	}
}

function virtualstock_backup(&$backup_fields)
{
	global $db, $table;
	$backup_fields[] = 'virtualstock';
}

/*************************************************************************************/
// ADMIN PART
/*************************************************************************************/

function virtualstock_admin_newpoints_menu(&$sub_menu)
{
	global $lang;
	
	newpoints_lang_load('virtualstock');
	$sub_menu[] = array('id' => 'virtualstock', 'title' => $lang->virtualstock, 'link' => 'index.php?module=newpoints-virtualstock');
}

function virtualstock_admin_newpoints_action_handler(&$actions)
{
	$actions['virtualstock'] = array('active' => 'virtualstock', 'file' => 'virtualstock');
}

function virtualstock_admin_permissions(&$admin_permissions)
{
  	global $db, $mybb, $lang;
  
	newpoints_lang_load('virtualstock');
	$admin_permissions['virtualstock'] = $lang->virtualstock_canmanage;
	
}

function virtualstock_messageredirect($message, $error=0, $action='')
{
  	global $db, $mybb, $lang;
	
	if (!$message)
		return;
		
	if ($action)
		$parameters = '&amp;action='.$action;
  
	if ($error)
	{
		flash_message($message, 'error');
		admin_redirect("index.php?module=newpoints-virtualstock".$parameters);
	}
	else {
		flash_message($message, 'success');
		admin_redirect("index.php?module=newpoints-virtualstock".$parameters);
	}
}

function virtualstock_admin()
{
	global $db, $lang, $mybb, $page, $run_module, $action_file, $mybbadmin, $plugins;
	
	newpoints_lang_load('virtualstock');
	
	if($run_module == 'newpoints' && $action_file == 'virtualstock')
	{
		if ($mybb->request_method == "post")
		{
			switch ($mybb->input['action'])
			{
				case 'do_addcat':
					if ($mybb->input['name'] == '')
					{
						virtualstock_messageredirect($lang->virtualstock_missing_field, 1);
					}
					
					$name = $db->escape_string($mybb->input['name']);
					$description = $db->escape_string($mybb->input['description']);
					
					// get visible to user groups options
					if(is_array($mybb->input['usergroups']))
					{
						foreach($mybb->input['usergroups'] as $gid)
						{
							if($gid == $mybb->input['usergroups'])
							{
								unset($mybb->input['usergroups'][$gid]);
							}
						}
						$usergroups = implode(",", $mybb->input['usergroups']);
					}
					else
					{
						$usergroups = '';
					}
					
					$usergroups = $db->escape_string($usergroups);
					$visible = intval($mybb->input['visible']);
					$icon = $db->escape_string($mybb->input['icon']);
					$disporder = intval($mybb->input['disporder']);
					$expanded = intval($mybb->input['expanded']);
					
					$insert_query = array('name' => $name, 'description' => $description, 'usergroups' => $usergroups, 'visible' => $visible, 'disporder' => $disporder, 'icon' => $icon, 'expanded' => $expanded);
					$db->insert_query('virtualstock_categories', $insert_query);
					
					virtualstock_messageredirect($lang->virtualstock_cat_added);
				break;
				case 'do_editcat':
					$cid = intval($mybb->input['cid']);
					if ($cid <= 0 || (!($cat = $db->fetch_array($db->simple_select('virtualstock_categories', '*', "cid = $cid")))))
					{
						virtualstock_messageredirect($lang->virtualstock_invalid_cat, 1);
					}
				
					if ($mybb->input['name'] == '')
					{
						virtualstock_messageredirect($lang->virtualstock_missing_field, 1);
					}
					
					$name = $db->escape_string($mybb->input['name']);
					$description = $db->escape_string($mybb->input['description']);
					// get visible to user groups options
					if(is_array($mybb->input['usergroups']))
					{
						foreach($mybb->input['usergroups'] as $gid)
						{
							if($gid == $mybb->input['usergroups'])
							{
								unset($mybb->input['usergroups'][$gid]);
							}
						}
						$usergroups = implode(",", $mybb->input['usergroups']);
					}
					else
					{
						$usergroups = '';
					}
					
					$usergroups = $db->escape_string($usergroups);
					$visible = intval($mybb->input['visible']);
					$icon = $db->escape_string($mybb->input['icon']);
					$disporder = intval($mybb->input['disporder']);
					$expanded = intval($mybb->input['expanded']);
					
					$update_query = array('name' => $name, 'description' => $description, 'usergroups' => $usergroups, 'visible' => $visible, 'disporder' => $disporder, 'icon' => $icon, 'expanded' => $expanded);
					$db->update_query('virtualstock_categories', $update_query, 'cid=\''.$cid.'\'');
					
					virtualstock_messageredirect($lang->virtualstock_cat_edited);
				break;
				
				case 'do_addcompany':
					if ($mybb->input['name'] == '' || $mybb->input['cid'] == '')
					{
						virtualstock_messageredirect($lang->virtualstock_missing_field, 1);
					}
					
					$name = $db->escape_string($mybb->input['name']);
					$description = $db->escape_string($mybb->input['description']);
					$icon = $db->escape_string($mybb->input['icon']);
					$total = intval($mybb->input['total']);
					$available = intval($mybb->input['available']);
					$price = floatval($mybb->input['price']);
					$minprice = floatval($mybb->input['minprice']);
					$maxprice = floatval($mybb->input['maxprice']);
					
					$visible = intval($mybb->input['visible']);
					$disporder = intval($mybb->input['disporder']);
					
					$cid = intval($mybb->input['cid']);
					if ($cid <= 0 || (!($cat = $db->fetch_array($db->simple_select('virtualstock_categories', '*', "cid = $cid")))))
					{
						virtualstock_messageredirect($lang->virtualstock_invalid_cat, 1);
					}
					
					$insert_array = array('name' => $name, 'description' => $description, 'icon' => $icon, 'visible' => $visible, 'disporder' => $disporder, 'price' => $price, 'minprice' => $minprice, 'maxprice' => $maxprice, 'total' => $total, 'cid' => $cid, 'available' => $available);
					
					$plugins->run_hooks("virtualstock_commit", $insert_array);
					
					$db->insert_query('virtualstock_companies', $insert_array);
					
					$db->write_query('UPDATE '.TABLE_PREFIX.'virtualstock_categories SET companies = companies+1 WHERE cid=\''.$cid.'\'');
					
					virtualstock_messageredirect($lang->virtualstock_company_added, 0, "companies&amp;cid=".$cid);
				break;
				case 'do_editcompany':
					$iid = intval($mybb->input['iid']);
					if ($iid <= 0 || (!($company = $db->fetch_array($db->simple_select('virtualstock_companies', '*', "iid = $iid")))))
					{
						virtualstock_messageredirect($lang->virtualstock_invalid_company, 1, 'companies');
					}
				
					if ($mybb->input['name'] == '' || $mybb->input['cid'] == '')
					{
						virtualstock_messageredirect($lang->virtualstock_missing_field, 1);
					}
					
					$name = $db->escape_string($mybb->input['name']);
					$description = $db->escape_string($mybb->input['description']);
					$icon = $db->escape_string($mybb->input['icon']);
					$total = intval($mybb->input['total']);
					$available = intval($mybb->input['available']);
					$price = floatval($mybb->input['price']);
					$minprice = floatval($mybb->input['minprice']);
					$maxprice = floatval($mybb->input['maxprice']);
					
					$visible = intval($mybb->input['visible']);
					$disporder = intval($mybb->input['disporder']);
					
					$cid = intval($mybb->input['cid']);
					if ($cid <= 0 || (!($cat = $db->fetch_array($db->simple_select('virtualstock_categories', '*', "cid = $cid")))))
					{
						virtualstock_messageredirect($lang->virtualstock_invalid_cat, 1);
					}
					
					$update_array = array('name' => $name, 'description' => $description, 'icon' => $icon, 'visible' => $visible, 'disporder' => $disporder, 'price' => $price, 'minprice' => $minprice, 'maxprice' => $maxprice, 'total' => $total, 'cid' => $cid, 'available' => $available);
					
					$plugins->run_hooks("virtualstock_commit", $update_array);
					
					$db->update_query('virtualstock_companies', $update_array, 'iid=\''.$iid.'\'');
					
					if ($cid != $company['cid'])
					{
						$db->write_query('UPDATE '.TABLE_PREFIX.'virtualstock_categories SET companies = companies-1 WHERE cid=\''.$company['cid'].'\'');
						$db->write_query('UPDATE '.TABLE_PREFIX.'virtualstock_categories SET companies = companies+1 WHERE cid=\''.$cid.'\'');
					}
					
					virtualstock_messageredirect($lang->virtualstock_company_edited, 0, "companies&amp;cid=".$cid);
				break;
			}
		}
		
		if ($mybb->input['action'] == 'do_deletecat')
		{
			$page->add_breadcrumb_item($lang->virtualstock, 'index.php?module=newpoints-virtualstock');
			$page->output_header($lang->virtualstock);
			
			$cid = intval($mybb->input['cid']);
		
			if($mybb->input['no']) // user clicked no
			{
				admin_redirect("index.php?module=newpoints-virtualstock");
			}
		
			if($mybb->request_method == "post")
			{
				if ($cid <= 0 || (!($cat = $db->fetch_array($db->simple_select('virtualstock_categories', 'cid', "cid = $cid")))))
				{
					virtualstock_messageredirect($lang->virtualstock_invalid_cat, 1);
				}
				
				$db->delete_query('virtualstock_categories', "cid = $cid");
				
				// unassign companies from this category
				$db->update_query('virtualstock_companies', array('cid' => 0), "cid = $cid");
				
				virtualstock_messageredirect($lang->virtualstock_cat_deleted);
			}
			else
			{
				$mybb->input['cid'] = intval($mybb->input['cid']);
				$form = new Form("index.php?module=newpoints-virtualstock&amp;action=do_deletecat&amp;cid={$mybb->input['cid']}&amp;my_post_key={$mybb->post_code}", 'post');
				echo "<div class=\"confirm_action\">\n";
				echo "<p>{$lang->virtualstock_confirm_deletecat}</p>\n";
				echo "<br />\n";
				echo "<p class=\"buttons\">\n";
				echo $form->generate_submit_button($lang->yes, array('class' => 'button_yes'));
				echo $form->generate_submit_button($lang->no, array("name" => "no", 'class' => 'button_no'));
				echo "</p>\n";
				echo "</div>\n";
				$form->end();
			}
		}
		elseif ($mybb->input['action'] == 'do_deletecompany')
		{
			$page->add_breadcrumb_item($lang->virtualstock, 'index.php?module=newpoints-virtualstock');
			$page->output_header($lang->virtualstock);
			
			$iid = intval($mybb->input['iid']);
		
			if($mybb->input['no']) // user clicked no
			{
				admin_redirect("index.php?module=newpoints-virtualstock", 0, "companies&amp;cid=".$cid);
			}
		
			if($mybb->request_method == "post")
			{
				if ($iid <= 0 || (!($company = $db->fetch_array($db->simple_select('virtualstock_companies', 'cid', "iid = $iid")))))
				{
					virtualstock_messageredirect($lang->virtualstock_invalid_company, 1, "companies&amp;cid=".$cid);
				}
				
				$db->delete_query('virtualstock_companies', "iid = $iid");
				
				// remove one from the companies count
				$db->write_query('UPDATE '.TABLE_PREFIX.'virtualstock_categories SET companies = companies-1 WHERE cid=\''.$company['cid'].'\'');
				
				virtualstock_messageredirect($lang->virtualstock_company_deleted, 0, "companies&amp;cid=".$cid);
			}
			else
			{
				$mybb->input['iid'] = intval($mybb->input['iid']);
				$form = new Form("index.php?module=newpoints-virtualstock&amp;action=do_deletecompany&amp;iid={$mybb->input['iid']}&amp;my_post_key={$mybb->post_code}", 'post');
				echo "<div class=\"confirm_action\">\n";
				echo "<p>{$lang->virtualstock_confirm_deletecompany}</p>\n";
				echo "<br />\n";
				echo "<p class=\"buttons\">\n";
				echo $form->generate_submit_button($lang->yes, array('class' => 'button_yes'));
				echo $form->generate_submit_button($lang->no, array("name" => "no", 'class' => 'button_no'));
				echo "</p>\n";
				echo "</div>\n";
				$form->end();
			}
		}
		
		if (!$mybb->input['action'] || $mybb->input['action'] == 'categories' || $mybb->input['action'] == 'inventory' || $mybb->input['action'] == 'addcat' || $mybb->input['action'] == 'editcat')
		{
			$page->add_breadcrumb_item($lang->virtualstock, 'index.php?module=newpoints-virtualstock');
						
			$page->output_header($lang->virtualstock);
				
			$sub_tabs['virtualstock_categories'] = array(
				'title'			=> $lang->virtualstock_categories,
				'link'			=> 'index.php?module=newpoints-virtualstock',
				'description'	=> $lang->virtualstock_categories_desc
			);
			
			if (!$mybb->input['action'] || $mybb->input['action'] == 'categories' || $mybb->input['action'] == 'addcat' || $mybb->input['action'] == 'editcat')
			{
				$sub_tabs['virtualstock_categories_add'] = array(
					'title'			=> $lang->virtualstock_addcat,
					'link'			=> 'index.php?module=newpoints-virtualstock&amp;action=addcat',
					'description'	=> $lang->virtualstock_addcat_desc
				);
				$sub_tabs['virtualstock_categories_edit'] = array(
					'title'			=> $lang->virtualstock_editcat,
					'link'			=> 'index.php?module=newpoints-virtualstock&amp;action=editcat',
					'description'	=> $lang->virtualstock_editcat_desc
				);
				$sub_tabs['virtualstock_categories_delete'] = array(
					'title'			=> $lang->virtualstock_deletecat,
					'link'			=> 'index.php?module=newpoints-virtualstock&amp;action=do_deletecat',
					'description'	=> $lang->virtualstock_deletecat_desc
				);
			}
		}
		
		if ($mybb->input['action'] == 'inventory')
		{
			$sub_tabs['virtualstock_inventory'] = array(
				'title'			=> $lang->virtualstock_inventory,
				'link'			=> 'index.php?module=newpoints-virtualstock&amp;action=inventory&amp;uid='.intval($mybb->input['uid']),
				'description'	=> $lang->virtualstock_inventory_desc
			);
		}
		
		if ($mybb->input['action'] == 'companies' || $mybb->input['action'] == 'addcompany' || $mybb->input['action'] == 'editcompany')
		{
			$page->add_breadcrumb_item($lang->virtualstock, 'index.php?module=newpoints-virtualstock');
						
			$page->output_header($lang->virtualstock);
			
			$sub_tabs['virtualstock_categories'] = array(
				'title'			=> $lang->virtualstock_categories,
				'link'			=> 'index.php?module=newpoints-virtualstock',
				'description'	=> $lang->virtualstock_categories_desc
			);
			
			$sub_tabs['virtualstock_companies'] = array(
				'title'			=> $lang->virtualstock_companies,
				'link'			=> 'index.php?module=newpoints-virtualstock&amp;action=companies&amp;cid='.intval($mybb->input['cid']),
				'description'	=> $lang->virtualstock_companies_desc
			);

			if ($mybb->input['action'] == 'companies' || $mybb->input['action'] == 'addcompany' || $mybb->input['action'] == 'editcompany')
			{
				$sub_tabs['virtualstock_companies_add'] = array(
					'title'			=> $lang->virtualstock_addcompany,
					'link'			=> 'index.php?module=newpoints-virtualstock&amp;action=addcompany&amp;cid='.intval($mybb->input['cid']),
					'description'	=> $lang->virtualstock_addcompany_desc
				);
				$sub_tabs['virtualstock_companies_edit'] = array(
					'title'			=> $lang->virtualstock_editcompany,
					'link'			=> 'index.php?module=newpoints-virtualstock&amp;action=editcompany',
					'description'	=> $lang->virtualstock_editcompany_desc
				);
				$sub_tabs['virtualstock_companies_delete'] = array(
					'title'			=> $lang->virtualstock_deletecompany,
					'link'			=> 'index.php?module=newpoints-virtualstock&amp;action=do_deletecompany',
					'description'	=> $lang->virtualstock_deletecompany_desc
				);
			}
		}
		
		if (!$mybb->input['action'] || $mybb->input['action'] == 'categories')
		{
			$page->output_nav_tabs($sub_tabs, 'virtualstock_categories');
			
			// table
			$table = new Table;
			$table->construct_header($lang->virtualstock_cat_name, array('width' => '30%'));
			$table->construct_header($lang->virtualstock_cat_description, array('width' => '35%'));
			$table->construct_header($lang->virtualstock_cat_companies, array('width' => '10%', 'class' => 'align_center'));
			$table->construct_header($lang->virtualstock_cat_disporder, array('width' => '10%', 'class' => 'align_center'));
			$table->construct_header($lang->virtualstock_cat_action, array('width' => '25%', 'class' => 'align_center'));
			
			$query = $db->simple_select('virtualstock_categories', '*', '', array('order_by' => 'disporder', 'order_dir' => 'ASC'));
			while ($cat = $db->fetch_array($query))
			{
				$table->construct_cell("<a href=\"index.php?module=newpoints-virtualstock&amp;action=companies&amp;cid={$cat['cid']}\">".htmlspecialchars_uni($cat['name'])."</a>");
				$table->construct_cell(htmlspecialchars_uni($cat['description']));
				$table->construct_cell(intval($cat['companies']), array('class' => 'align_center'));
				$table->construct_cell(intval($cat['disporder']), array('class' => 'align_center'));
				
				// actions column
				$table->construct_cell("<a href=\"index.php?module=newpoints-virtualstock&amp;action=editcat&amp;cid=".intval($cat['cid'])."\">".$lang->virtualstock_edit."</a> - <a href=\"index.php?module=newpoints-virtualstock&amp;action=do_deletecat&amp;cid=".intval($cat['cid'])."\">".$lang->virtualstock_delete."</a>", array('class' => 'align_center'));
				
				$table->construct_row();
			}
			
			if ($table->num_rows() == 0)
			{
				$table->construct_cell($lang->virtualstock_no_cats, array('colspan' => 5));
				$table->construct_row();
			}
			
			$table->output($lang->virtualstock_categories);
		}
		elseif ($mybb->input['action'] == 'addcat')
		{
			$page->output_nav_tabs($sub_tabs, 'virtualstock_categories_add');
			
			$query = $db->simple_select("usergroups", "gid, title", "gid != '1'", array('order_by' => 'title'));
			while($usergroup = $db->fetch_array($query))
			{
				$options[$usergroup['gid']] = $usergroup['title'];
			}
			
			$form = new Form("index.php?module=newpoints-virtualstock&amp;action=do_addcat", "post", "virtualstock");
	
			$form_container = new FormContainer($lang->virtualstock_addcat);
			$form_container->output_row($lang->virtualstock_addedit_cat_name."<em>*</em>", $lang->virtualstock_addedit_cat_name_desc, $form->generate_text_box('name', '', array('id' => 'name')), 'name');
			$form_container->output_row($lang->virtualstock_addedit_cat_description, $lang->virtualstock_addedit_cat_description_desc, $form->generate_text_box('description', '', array('id' => 'description')), 'description');
			$form_container->output_row($lang->virtualstock_addedit_cat_visible, $lang->virtualstock_addedit_cat_visible_desc, $form->generate_yes_no_radio('visible', 1, true, "", ""), 'visible');
			$form_container->output_row($lang->virtualstock_addedit_cat_icon, $lang->virtualstock_addedit_cat_icon_desc, $form->generate_text_box('icon', '', array('id' => 'icon')), 'icon');
			$form_container->output_row($lang->virtualstock_addedit_cat_usergroups, $lang->virtualstock_addedit_cat_usergroups_desc, $form->generate_select_box('usergroups[]', $options, '', array('id' => 'usergroups', 'multiple' => true, 'size' => 5)), 'groups');
			$form_container->output_row($lang->virtualstock_addedit_cat_disporder, $lang->virtualstock_addedit_cat_disporder_desc, $form->generate_text_box('disporder', '0', array('id' => 'disporder')), 'disporder');
			$form_container->output_row($lang->virtualstock_addedit_cat_expanded, $lang->virtualstock_addedit_cat_expanded_desc, $form->generate_yes_no_radio('expanded', 1, true, "", ""), 'expanded');
			
			$form_container->end();
			
			$buttons = "";
			$buttons[] = $form->generate_submit_button($lang->virtualstock_submit);
			$buttons[] = $form->generate_reset_button($lang->virtualstock_reset);
			$form->output_submit_wrapper($buttons);
			$form->end();
		}
		elseif ($mybb->input['action'] == 'editcat')
		{
			$page->output_nav_tabs($sub_tabs, 'virtualstock_categories_edit');
			
			$cid = intval($mybb->input['cid']);
			if ($cid <= 0 || (!($cat = $db->fetch_array($db->simple_select('virtualstock_categories', '*', "cid = $cid")))))
			{
				virtualstock_messageredirect($lang->virtualstock_invalid_cat, 1);
			}
			
			$query = $db->simple_select("usergroups", "gid, title", "gid != '1'", array('order_by' => 'title'));
			while($usergroup = $db->fetch_array($query))
			{
				$options[$usergroup['gid']] = $usergroup['title'];
			}
			
			$form = new Form("index.php?module=newpoints-virtualstock&amp;action=do_editcat", "post", "virtualstock");
			
			echo $form->generate_hidden_field('cid', $cat['cid']);
	
			$form_container = new FormContainer($lang->virtualstock_addcat);
			$form_container->output_row($lang->virtualstock_addedit_cat_name."<em>*</em>", $lang->virtualstock_addedit_cat_name_desc, $form->generate_text_box('name', htmlspecialchars_uni($cat['name']), array('id' => 'name')), 'name');
			$form_container->output_row($lang->virtualstock_addedit_cat_description, $lang->virtualstock_addedit_cat_description_desc, $form->generate_text_box('description', htmlspecialchars_uni($cat['description']), array('id' => 'description')), 'description');
			$form_container->output_row($lang->virtualstock_addedit_cat_visible, $lang->virtualstock_addedit_cat_visible_desc, $form->generate_yes_no_radio('visible', intval($cat['visible']), true, "", ""), 'visible');
			$form_container->output_row($lang->virtualstock_addedit_cat_icon, $lang->virtualstock_addedit_cat_icon_desc, $form->generate_text_box('icon', htmlspecialchars_uni($cat['icon']), array('id' => 'icon')), 'icon');
			$form_container->output_row($lang->virtualstock_addedit_cat_usergroups, $lang->virtualstock_addedit_cat_usergroups_desc, $form->generate_select_box('usergroups[]', $options, explode(',', $cat['usergroups']), array('id' => 'usergroups', 'multiple' => true, 'size' => 5)), 'groups');
			$form_container->output_row($lang->virtualstock_addedit_cat_disporder, $lang->virtualstock_addedit_cat_disporder_desc, $form->generate_text_box('disporder', intval($cat['disporder']), array('id' => 'disporder')), 'disporder');
			$form_container->output_row($lang->virtualstock_addedit_cat_expanded, $lang->virtualstock_addedit_cat_expanded_desc, $form->generate_yes_no_radio('expanded', intval($cat['expanded']), true, "", ""), 'expanded');
			
			$form_container->end();
			
			$buttons = "";
			$buttons[] = $form->generate_submit_button($lang->virtualstock_submit);
			$buttons[] = $form->generate_reset_button($lang->virtualstock_reset);
			$form->output_submit_wrapper($buttons);
			$form->end();
		}
		else if ($mybb->input['action'] == 'companies')
		{
			$page->output_nav_tabs($sub_tabs, 'virtualstock_companies');
			
			$cid = intval($mybb->input['cid']);
			if ($cid <= 0 || (!($cat = $db->fetch_array($db->simple_select('virtualstock_categories', '*', "cid = $cid")))))
			{
				virtualstock_messageredirect($lang->virtualstock_invalid_cat, 1);
			}
			
			// table
			$table = new Table;
			$table->construct_header($lang->virtualstock_company_icon, array('width' => '10%', 'class' => 'align_center'));
			$table->construct_header($lang->virtualstock_company_name, array('width' => '30%'));
			$table->construct_header($lang->virtualstock_company_price, array('width' => '15%', 'class' => 'align_center'));
			$table->construct_header($lang->virtualstock_company_disporder, array('width' => '15%', 'class' => 'align_center'));
			$table->construct_header($lang->virtualstock_company_action, array('width' => '20%', 'class' => 'align_center'));
			
			$query = $db->simple_select('virtualstock_companies', '*', 'cid=\''.$cid.'\'', array('order_by' => 'disporder', 'order_dir' => 'ASC'));
			
			while ($company = $db->fetch_array($query))
			{					
				if ($company['visible'] == 0)
					$visible_info = ' (<span style="color: #FF0000;">hidden</span>)';
				else
					$visible_info = '';
				
				$table->construct_cell(htmlspecialchars_uni($company['icon']) ? '<img src="'.$mybb->settings['bburl'].'/'.$company['icon'].'">' : '<img src="'.$mybb->settings['bburl'].'/images/newpoints/default.png">', array('class' => 'align_center'));
				$table->construct_cell(htmlspecialchars_uni($company['name'])." ".$visible_info."<br /><small>".htmlspecialchars_uni($company['description'])."</small>");
				$table->construct_cell(newpoints_format_points($company['price']), array('class' => 'align_center'));
				$table->construct_cell(intval($company['disporder']), array('class' => 'align_center'));
				
				// actions column
				$table->construct_cell("<a href=\"index.php?module=newpoints-virtualstock&amp;action=editcompany&amp;iid=".intval($company['iid'])."\">".$lang->virtualstock_edit."</a> - <a href=\"index.php?module=newpoints-virtualstock&amp;action=do_deletecompany&amp;iid=".intval($company['iid'])."\">".$lang->virtualstock_delete."</a>", array('class' => 'align_center'));
				
				$table->construct_row();
			}
			
			if ($table->num_rows() == 0)
			{
				$table->construct_cell($lang->virtualstock_no_companies, array('colspan' => 6));
				
				$table->construct_row();
			}
			
			$table->output($lang->virtualstock_companies);
		}
		elseif ($mybb->input['action'] == 'addcompany')
		{
			$page->output_nav_tabs($sub_tabs, 'virtualstock_companies_add');
			
			$cid = intval($mybb->input['cid']);
			if ($cid > 0)
			{
				if ($cid <= 0 || (!($cat = $db->fetch_array($db->simple_select('virtualstock_categories', '*', "cid = $cid")))))
				{
					virtualstock_messageredirect($lang->virtualstock_invalid_cat, 1);
				}
			}
			else
				$cid = 0;
			
			$categories[0] = $lang->virtualstock_select_cat;
			
			$query = $db->simple_select('virtualstock_categories', '*');
			while ($cat = $db->fetch_array($query))
				$categories[$cat['cid']] = $cat['name'];
			
			$form = new Form("index.php?module=newpoints-virtualstock&amp;action=do_addcompany", "post", "virtualstock");
	
			$form_container = new FormContainer($lang->virtualstock_addcompany);
			$form_container->output_row($lang->virtualstock_addedit_company_name."<em>*</em>", $lang->virtualstock_addedit_company_name_desc, $form->generate_text_box('name', '', array('id' => 'name')), 'name');
			$form_container->output_row($lang->virtualstock_addedit_company_description, $lang->virtualstock_addedit_company_description_desc, $form->generate_text_box('description', '', array('id' => 'description')), 'description');
			$form_container->output_row($lang->virtualstock_addedit_company_price, $lang->virtualstock_addedit_company_price_desc, $form->generate_text_box('price', '1', array('id' => 'price')), 'price');
			$form_container->output_row($lang->virtualstock_addedit_company_icon, $lang->virtualstock_addedit_company_icon_desc, $form->generate_text_box('icon', '', array('id' => 'icon')), 'icon');
			$form_container->output_row($lang->virtualstock_addedit_company_disporder, $lang->virtualstock_addedit_company_disporder_desc, $form->generate_text_box('disporder', '0', array('id' => 'disporder')), 'disporder');
			$form_container->output_row($lang->virtualstock_addedit_company_total, $lang->virtualstock_addedit_company_total_desc, $form->generate_text_box('total', 1000, array('id' => 'total')), 'total');
			$form_container->output_row($lang->virtualstock_addedit_company_available, $lang->virtualstock_addedit_company_available_desc, $form->generate_text_box('available', 1000, array('id' => 'available')), 'total');
			$form_container->output_row($lang->virtualstock_addedit_company_maxprice, '', $form->generate_text_box('maxprice', 5, array('id' => 'maxprice')), 'maxprice');
			$form_container->output_row($lang->virtualstock_addedit_company_minprice, '', $form->generate_text_box('minprice', 1, array('id' => 'minprice')), 'minprice');
			$form_container->output_row($lang->virtualstock_addedit_company_visible, $lang->virtualstock_addedit_company_visible_desc, $form->generate_yes_no_radio('visible', 1, true, "", ""), 'visible');
			$form_container->output_row($lang->virtualstock_addedit_company_category."<em>*</em>", $lang->virtualstock_addedit_company_category_desc, $form->generate_select_box('cid', $categories, $cid, array('id' => 'cid')), 'cid');

			$args = array($form_container, $form, array());
			$plugins->run_hooks("virtualstock_row", $args);
			
			$form_container->end();
			
			$buttons = "";
			$buttons[] = $form->generate_submit_button($lang->virtualstock_submit);
			$buttons[] = $form->generate_reset_button($lang->virtualstock_reset);
			$form->output_submit_wrapper($buttons);
			$form->end();
		}
		elseif ($mybb->input['action'] == 'editcompany')
		{
			$page->output_nav_tabs($sub_tabs, 'virtualstock_companies_edit');
			
			$iid = intval($mybb->input['iid']);
			if ($iid <= 0 || (!($company = $db->fetch_array($db->simple_select('virtualstock_companies', '*', "iid = $iid")))))
			{
				virtualstock_messageredirect($lang->virtualstock_invalid_company, 1, 'companies');
			}
			
			$categories[0] = $lang->virtualstock_select_cat;
			
			$query = $db->simple_select('virtualstock_categories', '*');
			while ($cat = $db->fetch_array($query))
				$categories[$cat['cid']] = $cat['name'];
			
			$form = new Form("index.php?module=newpoints-virtualstock&amp;action=do_editcompany", "post", "virtualstock");
			
			echo $form->generate_hidden_field('iid', $iid);
	
			$form_container = new FormContainer($lang->virtualstock_addcompany);
			$form_container->output_row($lang->virtualstock_addedit_company_name."<em>*</em>", $lang->virtualstock_addedit_company_name_desc, $form->generate_text_box('name', htmlspecialchars_uni($company['name']), array('id' => 'name')), 'name');
			$form_container->output_row($lang->virtualstock_addedit_company_description, $lang->virtualstock_addedit_company_description_desc, $form->generate_text_box('description', htmlspecialchars_uni($company['description']), array('id' => 'description')), 'description');
			$form_container->output_row($lang->virtualstock_addedit_company_price, $lang->virtualstock_addedit_company_price_desc, $form->generate_text_box('price', floatval($company['price']), array('id' => 'price')), 'price');
			$form_container->output_row($lang->virtualstock_addedit_company_icon, $lang->virtualstock_addedit_company_icon_desc, $form->generate_text_box('icon', htmlspecialchars_uni($company['icon']), array('id' => 'icon')), 'icon');
			$form_container->output_row($lang->virtualstock_addedit_company_disporder, $lang->virtualstock_addedit_company_disporder_desc, $form->generate_text_box('disporder', intval($company['disporder']), array('id' => 'disporder')), 'disporder');
			$form_container->output_row($lang->virtualstock_addedit_company_total, $lang->virtualstock_addedit_company_total_desc, $form->generate_text_box('total', intval($company['total']), array('id' => 'total')), 'total');
			$form_container->output_row($lang->virtualstock_addedit_company_available, $lang->virtualstock_addedit_company_available_desc, $form->generate_text_box('available', intval($company['available']), array('id' => 'available')), 'total');
			$form_container->output_row($lang->virtualstock_addedit_company_maxprice, '', $form->generate_text_box('maxprice', floatval($company['maxprice']), array('id' => 'maxprice')), 'maxprice');
			$form_container->output_row($lang->virtualstock_addedit_company_minprice, '', $form->generate_text_box('minprice', floatval($company['minprice']), array('id' => 'minprice')), 'minprice');
			$form_container->output_row($lang->virtualstock_addedit_company_visible, $lang->virtualstock_addedit_company_visible_desc, $form->generate_yes_no_radio('visible', intval($company['visible']), true, "", ""), 'visible');
			$form_container->output_row($lang->virtualstock_addedit_company_category."<em>*</em>", $lang->virtualstock_addedit_company_category_desc, $form->generate_select_box('cid', $categories, intval($company['cid']), array('id' => 'cid')), 'cid');

			$args = array($form_container, $form, $company);
			$plugins->run_hooks("virtualstock_row", $args);
			
			$form_container->end();
			
			$buttons = "";
			$buttons[] = $form->generate_submit_button($lang->virtualstock_submit);
			$buttons[] = $form->generate_reset_button($lang->virtualstock_reset);
			$form->output_submit_wrapper($buttons);
			$form->end();
		}
		
		$page->output_footer();
		exit;
	}
}

?>