<?php
/***************************************************************************
 *
 *   MyDownloads plugin (/admin/modules/mydownloads/manage_submissions.php)
 *	 Author: MyBB-Plugins.com
 *   Copyright:  2010-2011 MyBB-Plugins.com
 *   
 *   Website: http://www.mybb-plugins.com
 *
 *   Adds a subscriptions system to MyBB.
 *
 ***************************************************************************/

/****************************************************************************
* LICENCE: The licence file is named licence.txt and comes with this product.
* In case licence.txt does not come with this product, please contact 
* MyBB-Plugins.com immediately.
****************************************************************************/

// Disallow direct access to this file for security reasons
if(!defined("IN_MYBB"))
{
	die("Direct initialization of this file is not allowed.<br /><br />Please make sure IN_MYBB is defined.");
}

$lang->load('mydownloads');

// check if NewPoints is installed
$plugins_cache = $cache->read("plugins");
if(isset($plugins_cache['active']['newpoints']) && $mybb->settings['mydownloads_bridge_newpoints'] == 1)
	$newpoints_installed = true;
else
	$newpoints_installed = false;
	
if ($mybb->settings['mydownloads_paypal_enabled'] == 1)
	$paypal_enabled = true;
else
	$paypal_enabled = false;

if ($mybb->input['action'] == 'view')
{
	$query = $db->simple_select('mydownloads_submissions', '*', 'sid='.intval($mybb->input['sid']));
	$submission = $db->fetch_array($query);
	
	?>
	<div class="modal">
	<div style="overflow-y: auto; max-height: 400px;">
	<?php
	
	if($submission['preview'] != '')
	{
		$submission['preview'] = unserialize($submission['preview']);
		
		// Take the first image as cover
		$submission['preview'] = $submission['preview'][0];
	}
	else
	{
		$submission['preview'] = 'nopreview.png';
	}
		
	if (!$submission['license'])
		$submission['license'] = $lang->mydownloads_no_license_set;
		
	if (!$submission['version'])
		$submission['version'] = $lang->mydownloads_no_version_set;
		
	$table = new Table();

	$table->construct_cell($lang->mydownloads_submission_dl_name.':');
	$table->construct_cell(htmlspecialchars_uni($submission['name']));
	$table->construct_row();

	if (empty($submission['description']))
		$submission['description'] = $lang->mydownloads_no_desc_set;
	$table->construct_cell($lang->mydownloads_submission_dl_desc.':');
	$table->construct_cell(htmlspecialchars_uni($submission['description']));
	$table->construct_row();

	$table->construct_cell($lang->mydownloads_submission_dl_preview.':');
	$table->construct_cell('<a href="'.$mybb->settings['bburl'].'/'.$mybb->settings['mydownloads_previews_dir'].'/'.$submission['preview'].'" target="_blank"><img src="'.$mybb->settings['bburl'].'/'.$mybb->settings['mydownloads_previews_dir'].'/'.$submission['preview'].'" width="50" height="50"></a>');
	$table->construct_row();

	if($submission['url'] == '')
	{
		$table->construct_cell($lang->mydownloads_submission_dl_download.':');
		$table->construct_cell('<a href="'.$mybb->settings['bburl'].'/'.$mybb->settings['mydownloads_downloads_dir'].'/'.$submission['download'].'" target="_blank">'.$lang->mydownloads_download_file.'</a>');
		$table->construct_row();
	}
	
	if($submission['url'] != '')
	{
		$table->construct_cell($lang->mydownloads_submission_dl_download.':');
		$table->construct_cell(nl2br(htmlspecialchars_uni($submission['url'])));
		$table->construct_row();
	}
	
	$table->construct_cell($lang->mydownloads_submission_dl_license.':');
	$table->construct_cell(htmlspecialchars_uni($submission['license']));
	$table->construct_row();
	
	$table->construct_cell($lang->mydownloads_submission_dl_version.':');
	$table->construct_cell(htmlspecialchars_uni($submission['version']));
	$table->construct_row();
	
	if($submission['url'] == '')
	{
		$table->construct_cell($lang->mydownloads_submission_dl_filetype.':');
		$table->construct_cell(htmlspecialchars_uni($submission['filetype']));
		$table->construct_row();
		
		$table->construct_cell($lang->mydownloads_submission_dl_filesize.':');
		$table->construct_cell(get_friendly_size($submission['filesize']));
		$table->construct_row();
	}
		
	$table->construct_cell($lang->mydownloads_submission_dl_cat.':');
	$table->construct_cell(htmlspecialchars_uni($db->fetch_field($db->simple_select('mydownloads_categories', 'name', 'cid='.intval($submission['cid'])), 'name')));
	$table->construct_row();
	
	if ($paypal_enabled) {
		$table->construct_cell($lang->mydownloads_submission_dl_price.':');
		$table->construct_cell($submission['price']." ".$mybb->settings['mydownloads_paypal_currency']);
		$table->construct_row();
		
		if($submission['receiver_email'] == '')
			$submission['receiver_email'] = $lang->mydownloads_default;
		
		$table->construct_cell($lang->mydownloads_submission_dl_email.':');
		$table->construct_cell(htmlspecialchars_uni($submission['receiver_email']));
		$table->construct_row();
	}

	
	if ($newpoints_installed) {
		$table->construct_cell($lang->mydownloads_submission_dl_points.':');
		$table->construct_cell(newpoints_format_points($submission['points']));
		$table->construct_row();
	}

	$table->output($lang->sprintf($lang->mydownloads_viewing_download, $submission['name']));
		
	?>
	</div>
	</div>
	<?php
	
	exit;
}

$page->add_breadcrumb_item($lang->mydownloads_manage_submissions, 'index.php?module=mydownloads-manage_submissions');

$page->output_header($lang->mydownloads_manage_submissions);

if (!$mybb->input['action']) {
	
	$per_page = 10;
	if($mybb->input['page'] && intval($mybb->input['page']) > 1)
	{
		$mybb->input['page'] = intval($mybb->input['page']);
		$start = ($mybb->input['page']*$per_page)-$per_page;
	}
	else
	{
		$mybb->input['page'] = 1;
		$start = 0;
	}
	
	$query = $db->simple_select("mydownloads_submissions", "COUNT(sid) as submissions");
	$total_rows = $db->fetch_field($query, "submissions");

	echo "<br />".draw_admin_pagination($mybb->input['page'], $per_page, $total_rows, "index.php?module=mydownloads-manage_submissions&amp;page={page}");
	
	$field = array();
	
	// table
	$table = new Table;
	$table->construct_header($lang->mydownloads_submission_username, array('width' => '30%'));
	$table->construct_header($lang->mydownloads_submission_dl_name, array('width' => '20%'));
	$table->construct_header($lang->mydownloads_submission_dl_cat, array('width' => '20%'));
	$table->construct_header($lang->mydownloads_submission_dl_approve, array('width' => '10%'));
	$table->construct_header($lang->mydownloads_submission_dl_disapprove, array('width' => '10%'));
	$table->construct_header($lang->mydownloads_submission_dl_view, array('width' => '10%'));
	
	$query = $db->simple_select('mydownloads_submissions', '*', '', array('order_by' => 'sid', 'order_dir' => 'DESC', 'limit' => "{$start}, {$per_page}"));
	while($sub = $db->fetch_array($query)) {

		$submission = $sub;

		$table->construct_cell('<a href="'.$mybb->settings['bburl'].'/member.php?action=profile&amp;uid='.intval($submission['submitter_uid']).'" target="_blank">'.htmlspecialchars_uni($submission['submitter']).'</a>');
		$table->construct_cell(htmlspecialchars_uni($submission['name']));
		
		$category = $db->fetch_field($db->simple_select('mydownloads_categories', 'name', 'cid='.intval($submission['cid'])), 'name');
		$table->construct_cell(htmlspecialchars_uni($category));
		
		$form = new Form("index.php?module=mydownloads-manage_submissions&amp;action=approve", "post", 'mydownloads" onsubmit="return confirm(\''.mydownloads_jsspecialchars($lang->mydownloads_submission_dl_approve_confirm).'\');', 0, "", true);
		$html_data = $form->construct_return;
		$html_data .= $form->generate_hidden_field("sid", $submission['sid']);
		$html_data .= "<input type=\"submit\" class=\"submit_button\" value=\"{$lang->mydownloads_submission_dl_approve}\" />";
		$html_data .= $form->end();
		
		$table->construct_cell($html_data, array('width' => '10%'));
		
		$form = new Form("index.php?module=mydownloads-manage_submissions&amp;action=disapprove", "post", 'mydownloads" onsubmit="return confirm(\''.mydownloads_jsspecialchars($lang->mydownloads_submission_dl_disapprove_confirm).'\');', 0, "", true);
		$html_data = $form->construct_return;
		$html_data .= $form->generate_hidden_field("sid", $submission['sid']);
		$html_data .= "<input type=\"submit\" class=\"submit_button\" value=\"{$lang->mydownloads_submission_dl_disapprove}\" />";
		$html_data .= $form->end();
		
		$table->construct_cell($html_data, array('width' => '10%'));
		
		$form = new Form("", "post", 'mydownloads" onsubmit="MyBB.popupWindow(\'index.php?module=mydownloads-manage_submissions&amp;action=view&amp;sid='.$submission['sid'].'\', null, true); return false;', 0, "", true);
		$html_data = $form->construct_return;
		$html_data .= $form->generate_hidden_field("sid", $submission['sid']);
		$html_data .= "<input type=\"submit\" class=\"submit_button\" value=\"{$lang->mydownloads_submission_dl_view}\" />";
		$html_data .= $form->end();
		
		$table->construct_cell($html_data, array('width' => '10%'));
		
		$table->construct_row();
	}
	
	if (!$submission)
	{
		$table->construct_cell($lang->mydownloads_sumbissions_not_found, array('colspan' => 7));
		$table->construct_row();
	}
	
	$table->output($lang->mydownloads_downloads_submissions);
	
	$page->output_footer();
}
elseif ($mybb->input['action'] == 'approve') {
	
	if ((!($sid = intval($mybb->input['sid']))) || (!($submission = $db->fetch_array($db->simple_select('mydownloads_submissions', '*', 'sid='.intval($mybb->input['sid']), array('limit' => 1))))))
	{
		flash_message($lang->mydownloads_invalid_sid, 'error');
		admin_redirect("index.php?module=mydownloads-manage_submissions");
	}
	if ($mybb->request_method == "post")
	{
		if(!isset($mybb->input['my_post_key']) || $mybb->post_code != $mybb->input['my_post_key'])
		{
			$mybb->request_method = "get";
			flash_message($lang->mydownloads_error, 'error');
			admin_redirect("index.php?module=mydownloads-manage_submissions");
		}
		
		$cid = intval($submission['cid']);
		if ($cid <= 0 || (!($cat = $db->fetch_array($db->simple_select('mydownloads_categories', '*', "cid = $cid")))))
			error($lang->mydownloads_no_cid_dl);
		
		$lang->mydownloads_log_approved = $lang->sprintf($lang->mydownloads_log_approved, htmlspecialchars_uni($submission['submitter']), htmlspecialchars_uni($submission['name']));
		
		log_admin_action($lang->mydownloads_log_approved);
		
		if ($submission['update_did'] > 0)
		{
			$download = mydownloads_get_download(intval($submission['update_did']), 'preview,thumbnail,download');
			$submission['old_download'] = $download['download'];
			unset($download);
		}
		
		if($submission['preview'] != '')
		{
			$submission['preview'] = unserialize($submission['preview']);
		}
		
		mydownloads_approve_submission($submission, $cat, $sid);
		
		if ($mybb->settings['mydownloads_pm_on_managing'] == 1) {
			// send pm
			require_once(MYBB_ROOT."/inc/datahandlers/pm.php");
	
			$pmhandler = new PMDataHandler();
	
			$subject = $lang->mydownloads_pm_subject_approved;
			$message = $lang->sprintf($lang->mydownloads_pm_message_approved, htmlspecialchars_uni($submission['name']));
	
			$pm = array(
					"subject" => $db->escape_string($subject),
					"message" => $db->escape_string($message),
					"icon" => 0,
					"fromid" => intval($mybb->user['uid']),
					"do" => '',
					"pmid" => ''
			);
	
			$pm['toid'] = array($submission['submitter_uid']);
	
			$pm['options'] = array(
				"savecopy" => "no",
				"saveasdraft" => 0,
				"signature" => "no",
				"disablesmilies" => "no",
			);
	
			$pmhandler->admin_override = 1;
			$pmhandler->set_data($pm);
	
			if(!$pmhandler->validate_pm())
			{
				// error sending PM
				$error_pm = $lang->mydownloads_error_pm;
			}
			else
				$pminfo = $pmhandler->insert_pm();
		}
		
		flash_message($lang->mydownloads_approved, 'success');
		admin_redirect("index.php?module=mydownloads-manage_submissions");
	}
	else {
		
		$form = new Form("index.php?module=mydownloads-manage_submissions&amp;action=approve", 'post');
	
		echo $form->generate_hidden_field("sid", intval($mybb->input['sid']));
		
		echo "<div class=\"confirm_action\">\n";
		echo "<p>{$lang->mydownloads_submission_dl_approve_confirm}</p>\n";
		echo "<br />\n";
		echo "<p class=\"buttons\">\n";
		echo $form->generate_submit_button($lang->yes, array('class' => 'button_yes'));
		echo $form->generate_submit_button($lang->no, array("name" => "no", 'class' => 'button_no'));
		echo "</p>\n";
		echo "</div>\n";
		$form->end();
		
		$page->output_footer();
	}
}
elseif ($mybb->input['action'] == 'disapprove') {
	
	if ((!($sid = intval($mybb->input['sid']))) || (!($submission = $db->fetch_array($db->simple_select('mydownloads_submissions', '*', 'sid='.intval($mybb->input['sid']), array('limit' => 1))))))
	{
		flash_message($lang->mydownloads_invalid_sid, 'error');
		admin_redirect("index.php?module=mydownloads-manage_submissions");
	}
	if ($mybb->request_method == "post")
	{
		if(!isset($mybb->input['my_post_key']) || $mybb->post_code != $mybb->input['my_post_key'])
		{
			$mybb->request_method = "get";
			flash_message($lang->mydownloads_error, 'error');
			admin_redirect("index.php?module=mydownloads-manage_submissions");
		}
		
		// delete submission
		$db->delete_query('mydownloads_submissions', 'sid='.$sid, 1);
		
		@unlink(MYBB_ROOT.$mybb->settings['mydownloads_previews_dir']."/".$submission['preview']);
		@unlink(MYBB_ROOT.$mybb->settings['mydownloads_previews_dir']."/".'thumbnail_'.$submission['preview']);
		@unlink(MYBB_ROOT.$mybb->settings['mydownloads_downloads_dir']."/".$submission['download']);
		
		if ($submission['update_did'])
			$db->update_query('mydownloads_downloads', array('hidden' => 0), 'did='.intval($submission['update_did']));
		
		$lang->mydownloads_log_disapproved = $lang->sprintf($lang->mydownloads_log_disapproved, htmlspecialchars_uni($submission['submitter']), htmlspecialchars_uni($submission['name']));
		
		log_admin_action($lang->mydownloads_log_disapproved);
		
		if ($mybb->settings['mydownloads_pm_on_managing'] == 1) {
			
			// send pm
			require_once(MYBB_ROOT."/inc/datahandlers/pm.php");
	
			$pmhandler = new PMDataHandler();
	
			$subject = $lang->mydownloads_pm_subject_disapproved;
			$message = $lang->sprintf($lang->mydownloads_pm_message_disapproved, htmlspecialchars_uni($submission['name']));
	
			$pm = array(
					"subject" => $db->escape_string($subject),
					"message" => $db->escape_string($message),
					"icon" => 0,
					"fromid" => intval($mybb->user['uid']),
					"do" => '',
					"pmid" => ''
			);
	
			$pm['toid'] = array($submission['submitter_uid']);
	
			$pm['options'] = array(
				"savecopy" => "no",
				"saveasdraft" => 0,
				"signature" => "no",
				"disablesmilies" => "no",
			);
	
			$pmhandler->admin_override = 1;
			$pmhandler->set_data($pm);
	
			if(!$pmhandler->validate_pm())
			{
				// error sending PM
				$error_pm = $lang->mydownloads_error_pm;
			}
			else
				$pminfo = $pmhandler->insert_pm();
		}
		
		flash_message($lang->mydownloads_disapproved, 'success');
		admin_redirect("index.php?module=mydownloads-manage_submissions");
	}
	else {
		$form = new Form("index.php?module=mydownloads-manage_submissions&amp;action=disapprove", 'post');
	
		echo $form->generate_hidden_field("sid", intval($mybb->input['sid']));
		
		echo "<div class=\"confirm_action\">\n";
		echo "<p>{$lang->mydownloads_submission_dl_disapprove_confirm}</p>\n";
		echo "<br />\n";
		echo "<p class=\"buttons\">\n";
		echo $form->generate_submit_button($lang->yes, array('class' => 'button_yes'));
		echo $form->generate_submit_button($lang->no, array("name" => "no", 'class' => 'button_no'));
		echo "</p>\n";
		echo "</div>\n";
		$form->end();
		
		$page->output_footer();
	}
}

$page->output_footer();

?>
